@@ -8,11 +8,13 @@ import (
 	"testing"
 	"time"
 
-	. "github.com/elves/elvish/pkg/cli"
-	. "github.com/elves/elvish/pkg/cli/clitest"
-	"github.com/elves/elvish/pkg/cli/term"
-	"github.com/elves/elvish/pkg/sys"
-	"github.com/elves/elvish/pkg/ui"
+	. "src.elv.sh/pkg/cli"
+	. "src.elv.sh/pkg/cli/clitest"
+	"src.elv.sh/pkg/cli/term"
+	"src.elv.sh/pkg/cli/tk"
+	"src.elv.sh/pkg/sys"
+	"src.elv.sh/pkg/testutil"
+	"src.elv.sh/pkg/ui"
 )
 
 // Lifecycle aspects.
@@ -50,7 +52,7 @@ func TestReadCode_ResetsStateBeforeReturning(t *testing.T) {
 
 	f.Stop()
 
-	if code := GetCodeBuffer(f.App); code != (CodeBuffer{}) {
+	if code := f.App.ActiveWidget().(tk.CodeArea).CopyState().Buffer; code != (tk.CodeBuffer{}) {
 		t.Errorf("Editor state has CodeBuffer %v, want empty", code)
 	}
 }
@@ -107,7 +109,7 @@ func TestReadCode_CallsAfterReadline(t *testing.T) {
 func TestReadCode_FinalRedraw(t *testing.T) {
 	f := Setup(WithSpec(func(spec *AppSpec) {
 		spec.CodeAreaState.Buffer.Content = "code"
-		spec.State.Addon = Label{Content: ui.T("addon")}
+		spec.State.Addons = []tk.Widget{tk.Label{Content: ui.T("addon")}}
 	}))
 
 	// Wait until the stable state.
@@ -332,49 +334,146 @@ func TestReadCode_HidesRPromptInFinalRedrawIfNotPersistent(t *testing.T) {
 
 // Addon.
 
-func TestReadCode_LetsAddonHandleEvents(t *testing.T) {
+func TestReadCode_LetsLastWidgetHandleEvents(t *testing.T) {
 	f := Setup(WithSpec(func(spec *AppSpec) {
-		spec.State.Addon = NewCodeArea(CodeAreaSpec{
-			Prompt: func() ui.Text { return ui.T("addon> ") },
-		})
+		spec.State.Addons = []tk.Widget{
+			tk.NewCodeArea(tk.CodeAreaSpec{
+				Prompt: func() ui.Text { return ui.T("addon1> ") },
+			}),
+			tk.NewCodeArea(tk.CodeAreaSpec{
+				Prompt: func() ui.Text { return ui.T("addon2> ") },
+			}),
+		}
 	}))
 	defer f.Stop()
 
 	feedInput(f.TTY, "input")
 
 	wantBuf := bb().Newline(). // empty main code area
-					Write("addon> input").SetDotHere(). // addon
+					Write("addon1> ").Newline().         // addon1 did not handle inputs
+					Write("addon2> input").SetDotHere(). // addon2 handled inputs
 					Buffer()
 	f.TTY.TestBuffer(t, wantBuf)
 }
 
+func TestReadCode_PutsCursorOnLastWidgetWithFocus(t *testing.T) {
+	f := Setup(WithSpec(func(spec *AppSpec) {
+		spec.State.Addons = []tk.Widget{
+			testAddon{tk.Label{Content: ui.T("addon1> ")}, true},
+			testAddon{tk.Label{Content: ui.T("addon2> ")}, false},
+		}
+	}))
+	defer f.Stop()
+
+	f.TestTTY(t, "\n", // main code area is empty
+		term.DotHere, "addon1> ", "\n", // addon 1 has focus
+		"addon2> ", // addon 2 has no focus
+	)
+}
+
+func TestPushAddonPopAddon(t *testing.T) {
+	f := Setup()
+	defer f.Stop()
+
+	f.TestTTY(t /* nothing */)
+
+	f.App.PushAddon(tk.Label{Content: ui.T("addon1> ")})
+	f.App.Redraw()
+	f.TestTTY(t, "\n",
+		term.DotHere, "addon1> ")
+
+	f.App.PushAddon(tk.Label{Content: ui.T("addon2> ")})
+	f.App.Redraw()
+	f.TestTTY(t, "\n",
+		"addon1> \n",
+		term.DotHere, "addon2> ")
+
+	f.App.PopAddon()
+	f.App.Redraw()
+	f.TestTTY(t, "\n",
+		term.DotHere, "addon1> ")
+
+	f.App.PopAddon()
+	f.App.Redraw()
+	f.TestTTY(t /* nothing */)
+
+	// Popping addon when there is no addon does nothing
+	f.App.PopAddon()
+	// Add something to the codearea to ensure that we're not just looking at
+	// the previous buffer
+	f.TTY.Inject(term.K(' '))
+	f.TestTTY(t, " ", term.DotHere)
+}
+
+func TestReadCode_HidesAddonsWhenNotEnoughSpace(t *testing.T) {
+	f := Setup(
+		func(spec *AppSpec, tty TTYCtrl) {
+			spec.State.Addons = []tk.Widget{
+				tk.Label{Content: ui.T("addon1> ")},
+				tk.Label{Content: ui.T("addon2> ")}, // no space for this
+			}
+			tty.SetSize(2, 40)
+		})
+	defer f.Stop()
+
+	f.TestTTY(t,
+		"addon1> \n",
+		term.DotHere, "addon2> ")
+}
+
 type testAddon struct {
-	Empty
+	tk.Label
 	focus bool
 }
 
 func (a testAddon) Focus() bool { return a.focus }
 
-func TestReadCode_RespectsAddonFocusMethod(t *testing.T) {
-	addon := testAddon{}
-	f := Setup(WithSpec(func(spec *AppSpec) { spec.State.Addon = &addon }))
-	defer f.Stop()
+// Misc features.
 
-	wantBuf := bb().
-		SetDotHere(). // main code area has focus
-		Newline().Buffer()
-	f.TTY.TestBuffer(t, wantBuf)
-
-	addon.focus = true
-	f.App.Redraw()
-
-	wantBuf = bb().
-		Newline().SetDotHere(). // addon has focus
-		Buffer()
-	f.TTY.TestBuffer(t, wantBuf)
+func TestReadCode_UsesGlobalBindingsWithCodeAreaTarget(t *testing.T) {
+	testGlobalBindings(t, nil)
 }
 
-// Misc features.
+func TestReadCode_UsesGlobalBindingsWithAddonTarget(t *testing.T) {
+	testGlobalBindings(t, []tk.Widget{tk.Empty{}})
+}
+
+func testGlobalBindings(t *testing.T, addons []tk.Widget) {
+	gotWidgetCh := make(chan tk.Widget, 1)
+	f := Setup(WithSpec(func(spec *AppSpec) {
+		spec.GlobalBindings = tk.MapBindings{
+			term.K('X', ui.Ctrl): func(w tk.Widget) {
+				gotWidgetCh <- w
+			},
+		}
+		spec.State.Addons = addons
+	}))
+	defer f.Stop()
+
+	f.TTY.Inject(term.K('X', ui.Ctrl))
+	select {
+	case gotWidget := <-gotWidgetCh:
+		if gotWidget != f.App.ActiveWidget() {
+			t.Error("global binding not called with the active widget")
+		}
+	case <-time.After(testutil.Scaled(100 * time.Millisecond)):
+		t.Error("global binding not called")
+	}
+}
+
+func TestReadCode_DoesNotUseGlobalBindingsIfHandledByWidget(t *testing.T) {
+	f := Setup(WithSpec(func(spec *AppSpec) {
+		spec.GlobalBindings = tk.MapBindings{
+			term.K('a'): func(w tk.Widget) {},
+		}
+	}))
+	defer f.Stop()
+
+	f.TTY.Inject(term.K('a'))
+
+	// Still handled by code area instead of global binding
+	f.TestTTY(t, "a", term.DotHere)
+}
 
 func TestReadCode_TrimsBufferToMaxHeight(t *testing.T) {
 	f := Setup(func(spec *AppSpec, tty TTYCtrl) {
@@ -397,8 +496,8 @@ func TestReadCode_ShowNotes(t *testing.T) {
 	inHandler := make(chan struct{})
 	unblock := make(chan struct{})
 	f := Setup(WithSpec(func(spec *AppSpec) {
-		spec.OverlayHandler = MapHandler{
-			term.K('a'): func() {
+		spec.CodeAreaBindings = tk.MapBindings{
+			term.K('a'): func(tk.Widget) {
 				inHandler <- struct{}{}
 				<-unblock
 			},
