@@ -3,7 +3,7 @@ package histutil
 import (
 	"errors"
 
-	"github.com/elves/elvish/pkg/store"
+	"src.elv.sh/pkg/store/storedefs"
 )
 
 // Store is an abstract interface for history store.
@@ -11,9 +11,9 @@ type Store interface {
 	// AddCmd adds a new command history entry and returns its sequence number.
 	// Depending on the implementation, the Store might respect cmd.Seq and
 	// return it as is, or allocate another sequence number.
-	AddCmd(cmd store.Cmd) (int, error)
+	AddCmd(cmd storedefs.Cmd) (int, error)
 	// AllCmds returns all commands kept in the store.
-	AllCmds() ([]store.Cmd, error)
+	AllCmds() ([]storedefs.Cmd, error)
 	// Cursor returns a cursor that iterating through commands with the given
 	// prefix. The cursor is initially placed just after the last command in the
 	// store.
@@ -29,7 +29,7 @@ type Cursor interface {
 	// Get returns the command the cursor is currently at, or any error if the
 	// cursor is in an invalid state. If the cursor is "over the edge", the
 	// error is ErrEndOfHistory.
-	Get() (store.Cmd, error)
+	Get() (storedefs.Cmd, error)
 }
 
 // ErrEndOfHistory is returned by Cursor.Get if the cursor is currently over the
