@@ -4,21 +4,19 @@ import (
 	"os"
 	"testing"
 
-	"github.com/elves/elvish/pkg/testutil"
+	"src.elv.sh/pkg/testutil"
 )
 
 func TestLsColors(t *testing.T) {
-	_, cleanup := testutil.InTestDir()
-	defer cleanup()
-	restoreLsColors := WithTestLsColors()
-	defer restoreLsColors()
+	testutil.InTempDir(t)
+	SetTestLsColors(t)
 
 	// Test both feature-based and extension-based coloring.
 
 	colorist := GetColorist()
 
 	os.Mkdir("dir", 0755)
-	create("a.png", 0644)
+	create("a.png")
 
 	wantDirStyle := "34"
 	if style := colorist.GetStyle("dir"); style != wantDirStyle {
