@@ -6,8 +6,8 @@ import (
 	"testing"
 	"time"
 
-	"github.com/elves/elvish/pkg/testutil"
-	"github.com/elves/elvish/pkg/ui"
+	"src.elv.sh/pkg/testutil"
+	"src.elv.sh/pkg/ui"
 )
 
 func TestPrompt_DefaultCompute(t *testing.T) {
@@ -30,7 +30,7 @@ func TestPrompt_StalePrompt(t *testing.T) {
 	prompt := New(Config{
 		Compute: compute,
 		StaleThreshold: func() time.Duration {
-			return testutil.ScaledMs(10)
+			return testutil.Scaled(10 * time.Millisecond)
 		},
 	})
 
@@ -87,8 +87,7 @@ func TestPrompt_Eagerness0(t *testing.T) {
 	testNoUpdate(t, prompt)
 
 	// No update even if pwd has changed.
-	_, cleanup := testutil.InTestDir()
-	defer cleanup()
+	testutil.InTempDir(t)
 	prompt.Trigger(false)
 	testNoUpdate(t, prompt)
 
@@ -112,8 +111,7 @@ func TestPrompt_Eagerness5(t *testing.T) {
 	testNoUpdate(t, prompt)
 
 	// Update because the pwd has changed.
-	_, cleanup := testutil.InTestDir()
-	defer cleanup()
+	testutil.InTempDir(t)
 	prompt.Trigger(false)
 	testUpdate(t, prompt, ui.T("2> "))
 }
@@ -177,7 +175,7 @@ func testNoUpdate(t *testing.T, p *Prompt) {
 	select {
 	case update := <-p.LateUpdates():
 		t.Errorf("unexpected update %v", update)
-	case <-time.After(testutil.ScaledMs(10)):
+	case <-time.After(testutil.Scaled(10 * time.Millisecond)):
 		// OK
 	}
 }
