@@ -1,3 +1,4 @@
+//go:build !windows && !plan9
 // +build !windows,!plan9
 
 package term
@@ -6,7 +7,7 @@ import (
 	"os"
 	"time"
 
-	"github.com/elves/elvish/pkg/ui"
+	"src.elv.sh/pkg/ui"
 )
 
 // reader reads terminal escape sequences and decodes them into events.
@@ -110,17 +111,17 @@ func readEvent(rd byteReaderWithTimeout) (event Event, err error) {
 				// Mouse event.
 				cb := readRune()
 				if cb == runeEndOfSeq {
-					badSeq("Incomplete mouse event")
+					badSeq("incomplete mouse event")
 					return
 				}
 				cx := readRune()
 				if cx == runeEndOfSeq {
-					badSeq("Incomplete mouse event")
+					badSeq("incomplete mouse event")
 					return
 				}
 				cy := readRune()
 				if cy == runeEndOfSeq {
-					badSeq("Incomplete mouse event")
+					badSeq("incomplete mouse event")
 					return
 				}
 				down := true
@@ -147,7 +148,7 @@ func readEvent(rd byteReaderWithTimeout) (event Event, err error) {
 					nums[cur] = nums[cur]*10 + int(r-'0')
 				case r == runeEndOfSeq:
 					// Incomplete CSI.
-					badSeq("Incomplete CSI")
+					badSeq("incomplete CSI")
 					return
 				default: // Treat as a terminator.
 					break CSISeq
