@@ -1,6 +1,6 @@
 // Package term provides functionality for working with terminals.
 package term
 
-import "github.com/elves/elvish/pkg/logutil"
+import "src.elv.sh/pkg/logutil"
 
 var logger = logutil.GetLogger("[cli/term] ")
