@@ -2,11 +2,13 @@ package daemon
 
 import (
 	"errors"
-	"net/rpc"
+	"net"
 	"sync"
 
-	"github.com/elves/elvish/pkg/daemon/internal/api"
-	"github.com/elves/elvish/pkg/store"
+	"src.elv.sh/pkg/daemon/daemondefs"
+	"src.elv.sh/pkg/daemon/internal/api"
+	"src.elv.sh/pkg/rpc"
+	"src.elv.sh/pkg/store/storedefs"
 )
 
 const retriesOnShutdown = 3
@@ -17,18 +19,6 @@ var (
 	ErrDaemonUnreachable = errors.New("daemon offline")
 )
 
-// Client represents a daemon client.
-type Client interface {
-	store.Store
-
-	ResetConn() error
-	Close() error
-
-	Pid() (int, error)
-	SockPath() string
-	Version() (int, error)
-}
-
 // Implementation of the Client interface.
 type client struct {
 	sockPath  string
@@ -38,7 +28,7 @@ type client struct {
 
 // NewClient creates a new Client instance that talks to the socket. Connection
 // creation is deferred to the first request.
-func NewClient(sockPath string) Client {
+func NewClient(sockPath string) daemondefs.Client {
 	return &client{sockPath, nil, sync.WaitGroup{}}
 }
 
@@ -72,7 +62,7 @@ func (c *client) call(f string, req, res interface{}) error {
 
 	for attempt := 0; attempt < retriesOnShutdown; attempt++ {
 		if c.rpcClient == nil {
-			conn, err := dial(c.sockPath)
+			conn, err := net.Dial("unix", c.sockPath)
 			if err != nil {
 				return err
 			}
@@ -137,32 +127,25 @@ func (c *client) Cmd(seq int) (string, error) {
 	return res.Text, err
 }
 
-func (c *client) Cmds(from, upto int) ([]string, error) {
-	req := &api.CmdsRequest{From: from, Upto: upto}
-	res := &api.CmdsResponse{}
-	err := c.call("Cmds", req, res)
-	return res.Cmds, err
-}
-
-func (c *client) CmdsWithSeq(from, upto int) ([]store.Cmd, error) {
+func (c *client) CmdsWithSeq(from, upto int) ([]storedefs.Cmd, error) {
 	req := &api.CmdsWithSeqRequest{From: from, Upto: upto}
 	res := &api.CmdsWithSeqResponse{}
 	err := c.call("CmdsWithSeq", req, res)
 	return res.Cmds, err
 }
 
-func (c *client) NextCmd(from int, prefix string) (store.Cmd, error) {
+func (c *client) NextCmd(from int, prefix string) (storedefs.Cmd, error) {
 	req := &api.NextCmdRequest{From: from, Prefix: prefix}
 	res := &api.NextCmdResponse{}
 	err := c.call("NextCmd", req, res)
-	return store.Cmd{Text: res.Text, Seq: res.Seq}, err
+	return storedefs.Cmd{Text: res.Text, Seq: res.Seq}, err
 }
 
-func (c *client) PrevCmd(upto int, prefix string) (store.Cmd, error) {
+func (c *client) PrevCmd(upto int, prefix string) (storedefs.Cmd, error) {
 	req := &api.PrevCmdRequest{Upto: upto, Prefix: prefix}
 	res := &api.PrevCmdResponse{}
 	err := c.call("PrevCmd", req, res)
-	return store.Cmd{Text: res.Text, Seq: res.Seq}, err
+	return storedefs.Cmd{Text: res.Text, Seq: res.Seq}, err
 }
 
 func (c *client) AddDir(dir string, incFactor float64) error {
@@ -179,7 +162,7 @@ func (c *client) DelDir(dir string) error {
 	return err
 }
 
-func (c *client) Dirs(blacklist map[string]struct{}) ([]store.Dir, error) {
+func (c *client) Dirs(blacklist map[string]struct{}) ([]storedefs.Dir, error) {
 	req := &api.DirsRequest{Blacklist: blacklist}
 	res := &api.DirsResponse{}
 	err := c.call("Dirs", req, res)
