@@ -3,7 +3,7 @@ package diag
 import (
 	"testing"
 
-	"github.com/elves/elvish/pkg/tt"
+	"src.elv.sh/pkg/tt"
 )
 
 var Args = tt.Args
