@@ -5,12 +5,14 @@ import (
 	"errors"
 	"sort"
 
-	"github.com/elves/elvish/pkg/cli/addons/completion"
-	"github.com/elves/elvish/pkg/diag"
-	"github.com/elves/elvish/pkg/parse"
-	"github.com/elves/elvish/pkg/parse/parseutil"
+	"src.elv.sh/pkg/cli/modes"
+	"src.elv.sh/pkg/diag"
+	"src.elv.sh/pkg/parse"
+	"src.elv.sh/pkg/parse/parseutil"
 )
 
+type item = modes.CompletionItem
+
 // An error returned by Complete if the config has not supplied a PureEvaler.
 var errNoPureEvaler = errors.New("no PureEvaler supplied")
 
@@ -42,13 +44,13 @@ type ArgGenerator func(args []string) ([]RawItem, error)
 type Result struct {
 	Name    string
 	Replace diag.Ranging
-	Items   []completion.Item
+	Items   []modes.CompletionItem
 }
 
 // RawItem represents completion items before the quoting pass.
 type RawItem interface {
 	String() string
-	Cook(parse.PrimaryType) completion.Item
+	Cook(parse.PrimaryType) modes.CompletionItem
 }
 
 // PureEvaler encapsulates the functionality the completion algorithm needs from
@@ -63,7 +65,7 @@ type PureEvaler interface {
 	PurelyEvalPartialCompound(*parse.Compound, int) (string, bool)
 }
 
-// CodeBuffer is the same the type in github.com/elves/elvish/pkg/el/codearea,
+// CodeBuffer is the same the type in src.elv.sh/pkg/el/codearea,
 // replicated here to avoid an unnecessary dependency.
 type CodeBuffer struct {
 	Content string
@@ -84,7 +86,7 @@ func Complete(code CodeBuffer, cfg Config) (*Result, error) {
 	}
 
 	// Ignore the error; the function always returns a valid *ChunkNode.
-	tree, _ := parse.Parse(parse.Source{Name: "[interactive]", Code: code.Content})
+	tree, _ := parse.Parse(parse.Source{Name: "[interactive]", Code: code.Content}, parse.Config{})
 	leaf := parseutil.FindLeafNode(tree.Root, code.Dot)
 	for _, completer := range completers {
 		ctx, rawItems, err := completer(leaf, cfg)
@@ -92,7 +94,7 @@ func Complete(code CodeBuffer, cfg Config) (*Result, error) {
 			continue
 		}
 		rawItems = cfg.Filterer(ctx.name, ctx.seed, rawItems)
-		items := make([]completion.Item, len(rawItems))
+		items := make([]modes.CompletionItem, len(rawItems))
 		for i, rawCand := range rawItems {
 			items[i] = rawCand.Cook(ctx.quote)
 		}
@@ -105,8 +107,8 @@ func Complete(code CodeBuffer, cfg Config) (*Result, error) {
 	return nil, errNoCompletion
 }
 
-func dedup(items []completion.Item) []completion.Item {
-	var result []completion.Item
+func dedup(items []modes.CompletionItem) []modes.CompletionItem {
+	var result []modes.CompletionItem
 	for i, item := range items {
 		if i == 0 || item.ToInsert != items[i-1].ToInsert {
 			result = append(result, item)
