@@ -3,7 +3,7 @@ package complete
 import (
 	"reflect"
 
-	"github.com/elves/elvish/pkg/parse"
+	"src.elv.sh/pkg/parse"
 )
 
 // Reports whether a and b have the same dynamic type. Useful as a more succinct
@@ -16,7 +16,6 @@ func is(a, b parse.Node) bool {
 var (
 	aChunk    = &parse.Chunk{}
 	aPipeline = &parse.Pipeline{}
-	aForm     = &parse.Form{}
 	aArray    = &parse.Array{}
 	aIndexing = &parse.Indexing{}
 	aPrimary  = &parse.Primary{}
