@@ -1,9 +1,9 @@
 package complete
 
 import (
-	"github.com/elves/elvish/pkg/cli/addons/completion"
-	"github.com/elves/elvish/pkg/parse"
-	"github.com/elves/elvish/pkg/ui"
+	"src.elv.sh/pkg/cli/modes"
+	"src.elv.sh/pkg/parse"
+	"src.elv.sh/pkg/ui"
 )
 
 // PlainItem is a simple implementation of RawItem.
@@ -11,10 +11,10 @@ type PlainItem string
 
 func (p PlainItem) String() string { return string(p) }
 
-func (p PlainItem) Cook(q parse.PrimaryType) completion.Item {
+func (p PlainItem) Cook(q parse.PrimaryType) modes.CompletionItem {
 	s := string(p)
 	quoted, _ := parse.QuoteAs(s, q)
-	return completion.Item{ToInsert: quoted, ToShow: s}
+	return modes.CompletionItem{ToInsert: quoted, ToShow: s}
 }
 
 // noQuoteItem is a RawItem implementation that does not quote when cooked. This
@@ -23,9 +23,9 @@ type noQuoteItem string
 
 func (nq noQuoteItem) String() string { return string(nq) }
 
-func (nq noQuoteItem) Cook(parse.PrimaryType) completion.Item {
+func (nq noQuoteItem) Cook(parse.PrimaryType) modes.CompletionItem {
 	s := string(nq)
-	return completion.Item{ToInsert: s, ToShow: s}
+	return modes.CompletionItem{ToInsert: s, ToShow: s}
 }
 
 // ComplexItem is an implementation of RawItem that offers customization options.
@@ -38,13 +38,13 @@ type ComplexItem struct {
 
 func (c ComplexItem) String() string { return c.Stem }
 
-func (c ComplexItem) Cook(q parse.PrimaryType) completion.Item {
+func (c ComplexItem) Cook(q parse.PrimaryType) modes.CompletionItem {
 	quoted, _ := parse.QuoteAs(c.Stem, q)
 	display := c.Display
 	if display == "" {
 		display = c.Stem
 	}
-	return completion.Item{
+	return modes.CompletionItem{
 		ToInsert:  quoted + c.CodeSuffix,
 		ToShow:    display,
 		ShowStyle: c.DisplayStyle,
