@@ -3,30 +3,31 @@ package edit
 import (
 	"testing"
 
-	"github.com/elves/elvish/pkg/cli/term"
+	"src.elv.sh/pkg/cli/term"
+	"src.elv.sh/pkg/store/storedefs"
+	"src.elv.sh/pkg/ui"
 )
 
 func TestBeforeReadline(t *testing.T) {
-	f := setup(rc(
+	f := setup(t, rc(
 		`called = 0`,
 		`edit:before-readline = [ { called = (+ $called 1) } ]`))
-	defer f.Cleanup()
 
 	// Wait for UI to stabilize so that we can be sure that before-readline hooks
 	// have been called.
 	f.TestTTY(t, "~> ", term.DotHere)
 
-	testGlobal(t, f.Evaler, "called", 1.0)
+	testGlobal(t, f.Evaler, "called", 1)
 }
 
 func TestAfterReadline(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+	f := setup(t)
+
 	evals(f.Evaler,
 		`called = 0`,
 		`called-with = ''`,
 		`edit:after-readline = [
-	             [code]{ called = (+ $called 1); called-with = $code } ]`)
+		{|code| called = (+ $called 1); called-with = $code } ]`)
 
 	// Wait for UI to stabilize so that we can be sure that after-readline hooks
 	// are *not* called.
@@ -38,7 +39,7 @@ func TestAfterReadline(t *testing.T) {
 	f.Wait()
 
 	testGlobals(t, f.Evaler, map[string]interface{}{
-		"called":      1.0,
+		"called":      1,
 		"called-with": "test code",
 	})
 }
@@ -48,7 +49,7 @@ func TestAddCmdFilters(t *testing.T) {
 		name        string
 		rc          string
 		input       string
-		wantHistory []string
+		wantHistory []storedefs.Cmd
 	}{
 		// TODO: Enable the following two tests once error output can
 		// be tested.
@@ -60,43 +61,43 @@ func TestAddCmdFilters(t *testing.T) {
 		// },
 		// {
 		// 	name:        "callback outputs nothing",
-		// 	rc:          "edit:add-cmd-filters = [[_]{}]",
+		// 	rc:          "edit:add-cmd-filters = [{|_| }]",
 		// 	input:       "echo\n",
 		// 	wantHistory: []string{"echo"},
 		// },
 		{
 			name:        "callback outputs true",
-			rc:          "edit:add-cmd-filters = [[_]{ put $true }]",
+			rc:          "edit:add-cmd-filters = [{|_| put $true }]",
 			input:       "echo\n",
-			wantHistory: []string{"echo"},
+			wantHistory: []storedefs.Cmd{{Text: "echo", Seq: 1}},
 		},
 		{
 			name:        "callback outputs false",
-			rc:          "edit:add-cmd-filters = [[_]{ put $false }]",
+			rc:          "edit:add-cmd-filters = [{|_| put $false }]",
 			input:       "echo\n",
 			wantHistory: nil,
 		},
 		{
 			name:        "false-true chain",
-			rc:          "edit:add-cmd-filters = [[_]{ put $false } [_]{ put $true }]",
+			rc:          "edit:add-cmd-filters = [{|_| put $false } {|_| put $true }]",
 			input:       "echo\n",
 			wantHistory: nil,
 		},
 		{
 			name:        "true-false chain",
-			rc:          "edit:add-cmd-filters = [[_]{ put $true } [_]{ put $false }]",
+			rc:          "edit:add-cmd-filters = [{|_| put $true } {|_| put $false }]",
 			input:       "echo\n",
 			wantHistory: nil,
 		},
 		{
 			name:        "positive",
-			rc:          "edit:add-cmd-filters = [[cmd]{ ==s $cmd echo }]",
+			rc:          "edit:add-cmd-filters = [{|cmd| ==s $cmd echo }]",
 			input:       "echo\n",
-			wantHistory: []string{"echo"},
+			wantHistory: []storedefs.Cmd{{Text: "echo", Seq: 1}},
 		},
 		{
 			name:        "negative",
-			rc:          "edit:add-cmd-filters = [[cmd]{ ==s $cmd echo }]",
+			rc:          "edit:add-cmd-filters = [{|cmd| ==s $cmd echo }]",
 			input:       "echo x\n",
 			wantHistory: nil,
 		},
@@ -110,8 +111,7 @@ func TestAddCmdFilters(t *testing.T) {
 
 	for _, c := range cases {
 		t.Run(c.name, func(t *testing.T) {
-			f := setup(rc(c.rc))
-			defer f.Cleanup()
+			f := setup(t, rc(c.rc))
 
 			feedInput(f.TTYCtrl, c.input)
 			f.Wait()
@@ -122,14 +122,26 @@ func TestAddCmdFilters(t *testing.T) {
 }
 
 func TestAddCmdFilters_SkipsRemainingOnFalse(t *testing.T) {
-	f := setup(rc(
+	f := setup(t, rc(
 		`called = $false`,
-		`@edit:add-cmd-filters = [_]{ put $false } [_]{ called = $true; put $true }`,
+		`@edit:add-cmd-filters = {|_| put $false } {|_| called = $true; put $true }`,
 	))
-	defer f.Cleanup()
 
 	feedInput(f.TTYCtrl, "echo\n")
 	f.Wait()
 	testCommands(t, f.Store)
 	testGlobal(t, f.Evaler, "called", false)
 }
+
+func TestGlobalBindings(t *testing.T) {
+	f := setup(t, rc(
+		`var called = $false`,
+		`edit:global-binding[Ctrl-X] = { set called = $true }`,
+	))
+
+	f.TTYCtrl.Inject(term.K('X', ui.Ctrl))
+	f.TTYCtrl.Inject(term.K(ui.Enter))
+	f.Wait()
+
+	testGlobal(t, f.Evaler, "called", true)
+}
