@@ -6,24 +6,29 @@
 package edit
 
 import (
+	_ "embed"
 	"fmt"
 	"sync"
 
-	"github.com/elves/elvish/pkg/cli"
-	"github.com/elves/elvish/pkg/eval"
-	"github.com/elves/elvish/pkg/eval/vals"
-	"github.com/elves/elvish/pkg/eval/vars"
-	"github.com/elves/elvish/pkg/parse"
-	"github.com/elves/elvish/pkg/store"
+	"src.elv.sh/pkg/cli"
+	"src.elv.sh/pkg/eval"
+	"src.elv.sh/pkg/eval/vals"
+	"src.elv.sh/pkg/eval/vars"
+	"src.elv.sh/pkg/parse"
+	"src.elv.sh/pkg/store/storedefs"
 )
 
-// Editor is the interface line editor for Elvish.
+// Editor is the interactive line editor for Elvish.
 type Editor struct {
 	app cli.App
 	ns  *eval.Ns
 
 	excMutex sync.RWMutex
 	excList  vals.List
+
+	// Maybe move this to another type that represents the REPL cycle as a whole, not just the
+	// read/edit portion represented by the Editor type.
+	AfterCommand []func(src parse.Source, duration float64, err error)
 }
 
 // An interface that wraps notifyf and notifyError. It is only implemented by
@@ -34,23 +39,26 @@ type notifier interface {
 	notifyError(ctx string, e error)
 }
 
-// NewEditor creates a new editor from input and output terminal files.
-func NewEditor(tty cli.TTY, ev *eval.Evaler, st store.Store) *Editor {
+// NewEditor creates a new editor. The TTY is used for input and output. The
+// Evaler is used for syntax highlighting, completion, and calling callbacks.
+// The Store is used for saving and retrieving command and directory history.
+func NewEditor(tty cli.TTY, ev *eval.Evaler, st storedefs.Store) *Editor {
 	// Declare the Editor with a nil App first; some initialization functions
 	// require a notifier as an argument, but does not use it immediately.
 	ed := &Editor{excList: vals.EmptyList}
-	nb := eval.NsBuilder{}
+	nb := eval.BuildNsNamed("edit")
 	appSpec := cli.AppSpec{TTY: tty}
 
 	hs, err := newHistStore(st)
 	if err != nil {
-		// TODO(xiaq): Report the error.
+		_ = err // TODO(xiaq): Report the error.
 	}
 
 	initHighlighter(&appSpec, ev)
 	initMaxHeight(&appSpec, nb)
 	initReadlineHooks(&appSpec, ev, nb)
 	initAddCmdFilters(&appSpec, ev, nb, hs)
+	initGlobalBindings(&appSpec, ed, ev, nb)
 	initInsertAPI(&appSpec, ed, ev, nb)
 	initPrompts(&appSpec, ed, ev, nb)
 	ed.app = cli.NewApp(appSpec)
@@ -65,6 +73,7 @@ func NewEditor(tty cli.TTY, ev *eval.Evaler, st store.Store) *Editor {
 	initInstant(ed, ev, nb)
 	initMinibuf(ed, ev, nb)
 
+	initRepl(ed, ev, nb)
 	initBufferBuiltins(ed.app, nb)
 	initTTYBuiltins(ed.app, tty, nb)
 	initMiscBuiltins(ed.app, nb)
@@ -72,7 +81,7 @@ func NewEditor(tty cli.TTY, ev *eval.Evaler, st store.Store) *Editor {
 	initStoreAPI(ed.app, nb, hs)
 
 	ed.ns = nb.Ns()
-	evalDefaultBinding(ev, ed.ns)
+	initElvishState(ev, ed.ns)
 
 	return ed
 }
@@ -83,11 +92,15 @@ func NewEditor(tty cli.TTY, ev *eval.Evaler, st store.Store) *Editor {
 // examining tracebacks and other metadata.
 
 func initExceptionsAPI(ed *Editor, nb eval.NsBuilder) {
-	nb.Add("exceptions", vars.FromPtrWithMutex(&ed.excList, &ed.excMutex))
+	nb.AddVar("exceptions", vars.FromPtrWithMutex(&ed.excList, &ed.excMutex))
 }
 
-func evalDefaultBinding(ev *eval.Evaler, ns *eval.Ns) {
-	src := parse.Source{Name: "[default bindings]", Code: defaultBindingsElv}
+//go:embed init.elv
+var initElv string
+
+// Initialize the `edit` module by executing the pre-defined Elvish code for the module.
+func initElvishState(ev *eval.Evaler, ns *eval.Ns) {
+	src := parse.Source{Name: "[init.elv]", Code: initElv}
 	err := ev.Eval(src, eval.EvalCfg{Global: ns})
 	if err != nil {
 		panic(err)
@@ -99,7 +112,21 @@ func (ed *Editor) ReadCode() (string, error) {
 	return ed.app.ReadCode()
 }
 
+// Notify adds a note to the notification buffer.
+func (ed *Editor) Notify(note string) {
+	ed.app.Notify(note)
+}
+
+// RunAfterCommandHooks runs callbacks involving the interactive completion of a command line.
+func (ed *Editor) RunAfterCommandHooks(src parse.Source, duration float64, err error) {
+	for _, f := range ed.AfterCommand {
+		f(src, duration, err)
+	}
+}
+
 // Ns returns a namespace for manipulating the editor from Elvish code.
+//
+// See https://elv.sh/ref/edit.html for the Elvish API.
 func (ed *Editor) Ns() *eval.Ns {
 	return ed.ns
 }
