@@ -4,22 +4,20 @@ import (
 	"reflect"
 	"testing"
 
-	"github.com/elves/elvish/pkg/store"
+	"src.elv.sh/pkg/store/storedefs"
 )
 
 func TestEditor_AddsHistoryAfterAccepting(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+	f := setup(t)
 
 	feedInput(f.TTYCtrl, "echo x\n")
 	f.Wait()
 
-	testCommands(t, f.Store, "echo x")
+	testCommands(t, f.Store, storedefs.Cmd{Text: "echo x", Seq: 1})
 }
 
 func TestEditor_DoesNotAddEmptyCommandToHistory(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+	f := setup(t)
 
 	feedInput(f.TTYCtrl, "\n")
 	f.Wait()
@@ -27,9 +25,15 @@ func TestEditor_DoesNotAddEmptyCommandToHistory(t *testing.T) {
 	testCommands(t, f.Store /* no commands */)
 }
 
-func testCommands(t *testing.T, store store.Store, wantCmds ...string) {
+func TestEditor_Notify(t *testing.T) {
+	f := setup(t)
+	f.Editor.Notify("note")
+	f.TestTTYNotes(t, "note")
+}
+
+func testCommands(t *testing.T, store storedefs.Store, wantCmds ...storedefs.Cmd) {
 	t.Helper()
-	cmds, err := store.Cmds(0, 1024)
+	cmds, err := store.CmdsWithSeq(0, 1024)
 	if err != nil {
 		panic(err)
 	}
