@@ -4,9 +4,9 @@ package highlight
 import (
 	"time"
 
-	"github.com/elves/elvish/pkg/diag"
-	"github.com/elves/elvish/pkg/parse"
-	"github.com/elves/elvish/pkg/ui"
+	"src.elv.sh/pkg/diag"
+	"src.elv.sh/pkg/parse"
+	"src.elv.sh/pkg/ui"
 )
 
 // Config keeps configuration for highlighting code.
@@ -31,7 +31,7 @@ func highlight(code string, cfg Config, lateCb func(ui.Text)) (ui.Text, []error)
 	var errors []error
 	var errorRegions []region
 
-	tree, errParse := parse.Parse(parse.Source{Name: "[tty]", Code: code})
+	tree, errParse := parse.Parse(parse.Source{Name: "[tty]", Code: code}, parse.Config{})
 	if errParse != nil {
 		for _, err := range errParse.(*parse.Error).Entries {
 			if err.Context.From != len(code) {
