@@ -1,6 +1,6 @@
 package highlight
 
-import "github.com/elves/elvish/pkg/ui"
+import "src.elv.sh/pkg/ui"
 
 var stylingFor = map[string]ui.Styling{
 	barewordRegion:     nil,
