@@ -13,8 +13,8 @@ import (
 func eachVariableInTop(builtin, global *eval.Ns, ns string, f func(s string)) {
 	switch ns {
 	case "", ":":
-		global.IterateNames(f)
-		builtin.IterateNames(f)
+		global.IterateKeysString(f)
+		builtin.IterateKeysString(f)
 	case "e:":
 		fsutil.EachExternal(func(cmd string) {
 			f(cmd + eval.FnSuffix)
@@ -27,18 +27,18 @@ func eachVariableInTop(builtin, global *eval.Ns, ns string, f func(s string)) {
 		}
 	default:
 		segs := eval.SplitQNameSegs(ns)
-		mod := global.IndexName(segs[0])
+		mod := global.IndexString(segs[0])
 		if mod == nil {
-			mod = builtin.IndexName(segs[0])
+			mod = builtin.IndexString(segs[0])
 		}
 		for _, seg := range segs[1:] {
 			if mod == nil {
 				return
 			}
-			mod = mod.Get().(*eval.Ns).IndexName(seg)
+			mod = mod.Get().(*eval.Ns).IndexString(seg)
 		}
 		if mod != nil {
-			mod.Get().(*eval.Ns).IterateNames(f)
+			mod.Get().(*eval.Ns).IterateKeysString(f)
 		}
 	}
 }
@@ -49,13 +49,13 @@ func eachNsInTop(builtin, global *eval.Ns, f func(s string)) {
 	f("e:")
 	f("E:")
 
-	global.IterateNames(func(name string) {
+	global.IterateKeysString(func(name string) {
 		if strings.HasSuffix(name, eval.NsSuffix) {
 			f(name)
 		}
 	})
 
-	builtin.IterateNames(func(name string) {
+	builtin.IterateKeysString(func(name string) {
 		if strings.HasSuffix(name, eval.NsSuffix) {
 			f(name)
 		}
