@@ -1,3 +1,4 @@
+//go:build !windows && !plan9
 // +build !windows,!plan9
 
 package eval
@@ -9,9 +10,10 @@ import (
 	"strconv"
 	"syscall"
 
-	"github.com/elves/elvish/pkg/env"
-	"github.com/elves/elvish/pkg/eval/vals"
-	"github.com/elves/elvish/pkg/sys"
+	"src.elv.sh/pkg/env"
+	"src.elv.sh/pkg/eval/errs"
+	"src.elv.sh/pkg/eval/vals"
+	"src.elv.sh/pkg/sys/eunix"
 )
 
 // ErrNotInSameProcessGroup is thrown when the process IDs passed to fg are not
@@ -21,15 +23,19 @@ var ErrNotInSameProcessGroup = errors.New("not in the same process group")
 //elvdoc:fn exec
 //
 // ```elvish
-// exec $command?
+// exec $command? $args...
 // ```
 //
 // Replace the Elvish process with an external `$command`, defaulting to
-// `elvish`. This decrements `$E:SHLVL` before starting the new process.
+// `elvish`, passing the given arguments. This decrements `$E:SHLVL` before
+// starting the new process.
 //
 // This command always raises an exception on Windows with the message "not
 // supported on Windows".
 
+// Reference to syscall.Exec. Can be overridden in tests.
+var syscallExec = syscall.Exec
+
 func execFn(fm *Frame, args ...interface{}) error {
 	var argstrings []string
 	if len(args) == 0 {
@@ -50,7 +56,7 @@ func execFn(fm *Frame, args ...interface{}) error {
 	preExit(fm)
 	decSHLVL()
 
-	return syscall.Exec(argstrings[0], argstrings, os.Environ())
+	return syscallExec(argstrings[0], argstrings, os.Environ())
 }
 
 // Decrements $E:SHLVL. Called from execFn to ensure that $E:SHLVL remains the
@@ -65,7 +71,7 @@ func decSHLVL() {
 
 func fg(pids ...int) error {
 	if len(pids) == 0 {
-		return ErrArgs
+		return errs.ArityMismatch{What: "arguments", ValidLow: 1, ValidHigh: -1, Actual: len(pids)}
 	}
 	var thepgid int
 	for i, pid := range pids {
@@ -80,7 +86,7 @@ func fg(pids ...int) error {
 		}
 	}
 
-	err := sys.Tcsetpgrp(0, thepgid)
+	err := eunix.Tcsetpgrp(0, thepgid)
 	if err != nil {
 		return err
 	}
