@@ -1,3 +1,4 @@
+//go:build !windows && !plan9 && !js
 // +build !windows,!plan9,!js
 
 package eval_test
@@ -5,19 +6,17 @@ package eval_test
 import (
 	"testing"
 
-	. "github.com/elves/elvish/pkg/eval/evaltest"
+	. "src.elv.sh/pkg/eval/evaltest"
 )
 
-// Tests of the `builtin:has-external` command.
-func TestBuiltinHasExternal(t *testing.T) {
+func TestHasExternal(t *testing.T) {
 	Test(t,
 		That("has-external sh").Puts(true),
 		That("has-external random-invalid-command").Puts(false),
 	)
 }
 
-// Tests of the `builtin:search-external` command.
-func TestBuiltinSearchExternal(t *testing.T) {
+func TestSearchExternal(t *testing.T) {
 	Test(t,
 		// Even on UNIX systems we can't assume that commands like `sh` or
 		// `test` are in a specific directory. Those commands might be in /bin
@@ -37,8 +36,7 @@ func TestBuiltinSearchExternal(t *testing.T) {
 	)
 }
 
-// Tests of the `builtin:external` command.
-func TestBuiltinExternal(t *testing.T) {
+func TestExternal(t *testing.T) {
 	Test(t,
 		That(`(external sh) -c 'echo external-sh'`).Prints("external-sh\n"),
 	)
