@@ -6,7 +6,6 @@ import (
 	"math"
 	"math/big"
 	"sort"
-	"strconv"
 
 	"src.elv.sh/pkg/eval/errs"
 	"src.elv.sh/pkg/eval/vals"
@@ -40,7 +39,8 @@ func init() {
 
 		"keys": keys,
 
-		"order": order,
+		"compare": compare,
+		"order":   order,
 	})
 }
 
@@ -63,7 +63,7 @@ func init() {
 // ```
 
 func nsFn(m hashmap.Map) (*Ns, error) {
-	nb := make(NsBuilder)
+	nb := BuildNs()
 	for it := m.Iterator(); it.HasElem(); it.Next() {
 		k, v := it.Elem()
 		kstring, ok := k.(string)
@@ -72,7 +72,7 @@ func nsFn(m hashmap.Map) (*Ns, error) {
 				What:  `key of argument of "ns"`,
 				Valid: "string", Actual: vals.Kind(k)}
 		}
-		nb[kstring] = vars.FromInit(v)
+		nb.AddVar(kstring, vars.FromInit(v))
 	}
 	return nb.Ns(), nil
 }
@@ -137,12 +137,22 @@ func makeMap(input Inputs) (vals.Map, error) {
 //elvdoc:fn range
 //
 // ```elvish
-// range &step=1 $low? $high
+// range &step $start=0 $end
 // ```
 //
-// Output `$low`, `$low` + `$step`, ..., proceeding as long as smaller than
-// `$high` or until overflow. If not given, `$low` defaults to 0. The `$step`
-// must be positive.
+// Outputs numbers, starting from `$start` and ending before `$end`, using
+// `&step` as the increment.
+//
+// - If `$start` <= `$end`, `&step` defaults to 1, and `range` outputs values as
+//   long as they are smaller than `$end`. An exception is thrown if `&step` is
+//   given a negative value.
+//
+// - If `$start` > `$end`, `&step` defaults to -1, and `range` outputs values as
+//   long as they are greater than `$end`. An exception is thrown if `&step` is
+//   given a positive value.
+//
+// As a special case, if the outputs are floating point numbers, `range` also
+// terminates if the values stop changing.
 //
 // This command is [exactness-preserving](#exactness-preserving).
 //
@@ -150,14 +160,26 @@ func makeMap(input Inputs) (vals.Map, error) {
 //
 // ```elvish-transcript
 // ~> range 4
-// ▶ 0
-// ▶ 1
-// ▶ 2
-// ▶ 3
-// ~> range 1 6 &step=2
-// ▶ 1
-// ▶ 3
-// ▶ 5
+// ▶ (num 0)
+// ▶ (num 1)
+// ▶ (num 2)
+// ▶ (num 3)
+// ~> range 4 0
+// ▶ (num 4)
+// ▶ (num 3)
+// ▶ (num 2)
+// ▶ (num 1)
+// ~> range -3 3 &step=2
+// ▶ (num -3)
+// ▶ (num -1)
+// ▶ (num 1)
+// ~> range 3 -3 &step=-2
+// ▶ (num 3)
+// ▶ (num 1)
+// ▶ (num -1)
+// ~> range (- (math:pow 2 53) 1) +inf
+// ▶ (num 9007199254740991.0)
+// ▶ (num 9007199254740992.0)
 // ```
 //
 // When using floating-point numbers, beware that numerical errors can result in
@@ -185,96 +207,236 @@ func makeMap(input Inputs) (vals.Map, error) {
 
 type rangeOpts struct{ Step vals.Num }
 
-func (o *rangeOpts) SetDefaultOptions() { o.Step = 1 }
+// TODO: The default value can only be used implicitly; passing "range
+// &step=nil" results in an error.
+func (o *rangeOpts) SetDefaultOptions() { o.Step = nil }
 
 func rangeFn(fm *Frame, opts rangeOpts, args ...vals.Num) error {
 	var rawNums []vals.Num
 	switch len(args) {
 	case 1:
-		rawNums = []vals.Num{0, args[0], opts.Step}
+		rawNums = []vals.Num{0, args[0]}
 	case 2:
-		rawNums = []vals.Num{args[0], args[1], opts.Step}
+		rawNums = []vals.Num{args[0], args[1]}
 	default:
 		return errs.ArityMismatch{What: "arguments", ValidLow: 1, ValidHigh: 2, Actual: len(args)}
 	}
-	switch step := opts.Step.(type) {
-	case int:
-		if step <= 0 {
-			return errs.BadValue{
-				What: "step", Valid: "positive", Actual: strconv.Itoa(step)}
-		}
-	case *big.Int:
-		if step.Sign() <= 0 {
-			return errs.BadValue{
-				What: "step", Valid: "positive", Actual: step.String()}
-		}
-	case *big.Rat:
-		if step.Sign() <= 0 {
-			return errs.BadValue{
-				What: "step", Valid: "positive", Actual: step.String()}
-		}
-	case float64:
-		if step <= 0 {
-			return errs.BadValue{
-				What: "step", Valid: "positive", Actual: vals.ToString(step)}
-		}
+	if opts.Step != nil {
+		rawNums = append(rawNums, opts.Step)
 	}
 	nums := vals.UnifyNums(rawNums, vals.Int)
 
 	out := fm.ValueOutput()
+
 	switch nums := nums.(type) {
 	case []int:
-		lower, upper, step := nums[0], nums[1], nums[2]
-		for cur := lower; cur < upper; cur += step {
-			err := out.Put(vals.FromGo(cur))
-			if err != nil {
-				return err
-			}
-			if cur+step <= cur {
-				// Overflow
-				break
-			}
-		}
+		return rangeInt(nums, out)
 	case []*big.Int:
-		lower, upper, step := nums[0], nums[1], nums[2]
-		cur := &big.Int{}
-		for cur.Set(lower); cur.Cmp(upper) < 0; {
-			err := out.Put(vals.FromGo(cur))
-			if err != nil {
-				return err
-			}
-			next := &big.Int{}
-			next.Add(cur, step)
-			cur = next
-		}
+		return rangeBigInt(nums, out)
 	case []*big.Rat:
-		lower, upper, step := nums[0], nums[1], nums[2]
-		cur := &big.Rat{}
-		for cur.Set(lower); cur.Cmp(upper) < 0; {
-			err := out.Put(vals.FromGo(cur))
-			if err != nil {
-				return err
-			}
-			next := &big.Rat{}
-			next.Add(cur, step)
-			cur = next
-		}
+		return rangeBitRat(nums, out)
 	case []float64:
-		lower, upper, step := nums[0], nums[1], nums[2]
-		for cur := lower; cur < upper; cur += step {
-			err := out.Put(vals.FromGo(cur))
-			if err != nil {
-				return err
-			}
-			if cur+step <= cur {
-				// Overflow
-				break
-			}
-		}
+		return rangeFloat64(nums, out)
 	default:
 		panic("unreachable")
 	}
+}
 
+func rangeInt(nums []int, out ValueOutput) error {
+	start, end := nums[0], nums[1]
+	var step int
+	if start <= end {
+		if len(nums) == 3 {
+			step = nums[2]
+			if step <= 0 {
+				return errs.BadValue{
+					What: "step", Valid: "positive", Actual: vals.ToString(step)}
+			}
+		} else {
+			step = 1
+		}
+		for cur := start; cur < end; cur += step {
+			err := out.Put(vals.FromGo(cur))
+			if err != nil {
+				return err
+			}
+			if cur+step <= cur {
+				break
+			}
+		}
+	} else {
+		if len(nums) == 3 {
+			step = nums[2]
+			if step >= 0 {
+				return errs.BadValue{
+					What: "step", Valid: "negative", Actual: vals.ToString(step)}
+			}
+		} else {
+			step = -1
+		}
+		for cur := start; cur > end; cur += step {
+			err := out.Put(vals.FromGo(cur))
+			if err != nil {
+				return err
+			}
+			if cur+step >= cur {
+				break
+			}
+		}
+	}
+	return nil
+}
+
+// TODO: Use type parameters to deduplicate this with rangeInt when Elvish
+// requires Go 1.18.
+func rangeFloat64(nums []float64, out ValueOutput) error {
+	start, end := nums[0], nums[1]
+	var step float64
+	if start <= end {
+		if len(nums) == 3 {
+			step = nums[2]
+			if step <= 0 {
+				return errs.BadValue{
+					What: "step", Valid: "positive", Actual: vals.ToString(step)}
+			}
+		} else {
+			step = 1
+		}
+		for cur := start; cur < end; cur += step {
+			err := out.Put(vals.FromGo(cur))
+			if err != nil {
+				return err
+			}
+			if cur+step <= cur {
+				break
+			}
+		}
+	} else {
+		if len(nums) == 3 {
+			step = nums[2]
+			if step >= 0 {
+				return errs.BadValue{
+					What: "step", Valid: "negative", Actual: vals.ToString(step)}
+			}
+		} else {
+			step = -1
+		}
+		for cur := start; cur > end; cur += step {
+			err := out.Put(vals.FromGo(cur))
+			if err != nil {
+				return err
+			}
+			if cur+step >= cur {
+				break
+			}
+		}
+	}
+	return nil
+}
+
+var (
+	bigInt1    = big.NewInt(1)
+	bigIntNeg1 = big.NewInt(-1)
+)
+
+func rangeBigInt(nums []*big.Int, out ValueOutput) error {
+	start, end := nums[0], nums[1]
+	var step *big.Int
+	if start.Cmp(end) <= 0 {
+		if len(nums) == 3 {
+			step = nums[2]
+			if step.Sign() <= 0 {
+				return errs.BadValue{
+					What: "step", Valid: "positive", Actual: vals.ToString(step)}
+			}
+		} else {
+			step = bigInt1
+		}
+		var cur, next *big.Int
+		for cur = start; cur.Cmp(end) < 0; cur = next {
+			err := out.Put(vals.FromGo(cur))
+			if err != nil {
+				return err
+			}
+			next = &big.Int{}
+			next.Add(cur, step)
+			cur = next
+		}
+	} else {
+		if len(nums) == 3 {
+			step = nums[2]
+			if step.Sign() >= 0 {
+				return errs.BadValue{
+					What: "step", Valid: "negative", Actual: vals.ToString(step)}
+			}
+		} else {
+			step = bigIntNeg1
+		}
+		var cur, next *big.Int
+		for cur = start; cur.Cmp(end) > 0; cur = next {
+			err := out.Put(vals.FromGo(cur))
+			if err != nil {
+				return err
+			}
+			next = &big.Int{}
+			next.Add(cur, step)
+			cur = next
+		}
+	}
+	return nil
+}
+
+var (
+	bigRat1    = big.NewRat(1, 1)
+	bigRatNeg1 = big.NewRat(-1, 1)
+)
+
+// TODO: Use type parameters to deduplicate this with rangeBitInt when Elvish
+// requires Go 1.18.
+func rangeBitRat(nums []*big.Rat, out ValueOutput) error {
+	start, end := nums[0], nums[1]
+	var step *big.Rat
+	if start.Cmp(end) <= 0 {
+		if len(nums) == 3 {
+			step = nums[2]
+			if step.Sign() <= 0 {
+				return errs.BadValue{
+					What: "step", Valid: "positive", Actual: vals.ToString(step)}
+			}
+		} else {
+			step = bigRat1
+		}
+		var cur, next *big.Rat
+		for cur = start; cur.Cmp(end) < 0; cur = next {
+			err := out.Put(vals.FromGo(cur))
+			if err != nil {
+				return err
+			}
+			next = &big.Rat{}
+			next.Add(cur, step)
+			cur = next
+		}
+	} else {
+		if len(nums) == 3 {
+			step = nums[2]
+			if step.Sign() >= 0 {
+				return errs.BadValue{
+					What: "step", Valid: "negative", Actual: vals.ToString(step)}
+			}
+		} else {
+			step = bigRatNeg1
+		}
+		var cur, next *big.Rat
+		for cur = start; cur.Cmp(end) > 0; cur = next {
+			err := out.Put(vals.FromGo(cur))
+			if err != nil {
+				return err
+			}
+			next = &big.Rat{}
+			next.Add(cur, step)
+			cur = next
+		}
+	}
 	return nil
 }
 
@@ -742,20 +904,8 @@ func keys(fm *Frame, v interface{}) error {
 // argument. If the function throws an exception, `order` rethrows the exception
 // without outputting any value.
 //
-// If `&less-than` has value `$nil` (the default if not set), the following
-// comparison algorithm is used:
-//
-// - Numbers are compared numerically. For the sake of sorting, `NaN` is treated
-//   as smaller than all other numbers.
-//
-// - Strings are compared lexicographically by bytes, which is equivalent to
-//   comparing by codepoints under UTF-8.
-//
-// - Lists are compared lexicographically by elements, if the elements at the
-//   same positions are comparable.
-//
-// If the ordering between two elements are not defined by the conditions above,
-// no value is outputted and an exception is thrown.
+// If `&less-than` has value `$nil` (the default if not set), it is equivalent
+// to `{|a b| eq -1 (compare $a $b) }`.
 //
 // Examples:
 //
@@ -777,7 +927,7 @@ func keys(fm *Frame, v interface{}) error {
 // ▶ c
 // ▶ b
 // ▶ a
-// ~> order &less-than=[a b]{ eq $a x } [l x o r x e x m]
+// ~> order &less-than={|a b| eq $a x } [l x o r x e x m]
 // ▶ x
 // ▶ x
 // ▶ x
@@ -796,11 +946,13 @@ func keys(fm *Frame, v interface{}) error {
 // ▶ 1
 // ▶ 10
 // ▶ 5
-// ~> order &less-than=[a b]{ < $a $b } [5 1 10]
+// ~> order &less-than={|a b| < $a $b } [5 1 10]
 // ▶ 1
 // ▶ 5
 // ▶ 10
 // ```
+//
+// @cf compare
 
 type orderOptions struct {
 	Reverse  bool
@@ -809,10 +961,10 @@ type orderOptions struct {
 
 func (opt *orderOptions) SetDefaultOptions() {}
 
-// ErrUncomparable is raised by the order command when inputs contain
+// ErrUncomparable is raised by the compare and order commands when inputs contain
 // uncomparable values.
 var ErrUncomparable = errs.BadValue{
-	What:  `inputs to "order"`,
+	What:  `inputs to "compare" or "order"`,
 	Valid: "comparable values", Actual: "uncomparable values"}
 
 func order(fm *Frame, opts orderOptions, inputs Inputs) error {
@@ -855,12 +1007,12 @@ func order(fm *Frame, opts orderOptions, inputs Inputs) error {
 			return true
 		}
 	} else {
-		// Use default comparison implemented by compare.
+		// Use default comparison implemented by cmp.
 		lessFn = func(i, j int) bool {
 			if errSort != nil {
 				return true
 			}
-			o := compare(values[i], values[j])
+			o := cmp(values[i], values[j])
 			if o == uncomparable {
 				errSort = ErrUncomparable
 				return true
@@ -896,7 +1048,63 @@ const (
 	uncomparable
 )
 
-func compare(a, b interface{}) ordering {
+//elvdoc:fn compare
+//
+// ```elvish
+// compare $a $b
+// ```
+//
+// Outputs -1 if `$a` < `$b`, 0 if `$a` = `$b`, and 1 if `$a` > `$b`.
+//
+// The following comparison algorithm is used:
+//
+// - Typed numbers are compared numerically. The comparison is consistent with
+//   the [number comparison commands](#num-cmp), except that `NaN` values are
+//   considered equal to each other and smaller than all other numbers.
+//
+// - Strings are compared lexicographically by bytes, consistent with the
+//   [string comparison commands](#str-cmp). For UTF-8 encoded strings, this is
+//   equivalent to comparing by codepoints.
+//
+// - Lists are compared lexicographically by elements, if the elements at the
+//   same positions are comparable.
+//
+// If the ordering between two elements is not defined by the conditions above,
+// i.e. if the value of `$a` or `$b` is not covered by any of the cases above or
+// if they belong to different cases, a "bad value" exception is thrown.
+//
+// Examples:
+//
+// ```elvish-transcript
+// ~> compare a b
+// ▶ (num 1)
+// ~> compare b a
+// ▶ (num -1)
+// ~> compare x x
+// ▶ (num 0)
+// ~> compare (float64 10) (float64 1)
+// ▶ (num 1)
+// ```
+//
+// Beware that strings that look like numbers are treated as strings, not
+// numbers.
+//
+// @cf order
+
+func compare(fm *Frame, a, b interface{}) (int, error) {
+	switch cmp(a, b) {
+	case less:
+		return -1, nil
+	case equal:
+		return 0, nil
+	case more:
+		return 1, nil
+	default:
+		return 0, ErrUncomparable
+	}
+}
+
+func cmp(a, b interface{}) ordering {
 	switch a := a.(type) {
 	case int, *big.Int, *big.Rat, float64:
 		switch b.(type) {
@@ -922,8 +1130,7 @@ func compare(a, b interface{}) ordering {
 				return equal
 			case a < b:
 				return less
-			default:
-				// a > b
+			default: // a > b
 				return more
 			}
 		}
@@ -932,7 +1139,7 @@ func compare(a, b interface{}) ordering {
 			aIt := a.Iterator()
 			bIt := b.Iterator()
 			for aIt.HasElem() && bIt.HasElem() {
-				o := compare(aIt.Elem(), bIt.Elem())
+				o := cmp(aIt.Elem(), bIt.Elem())
 				if o != equal {
 					return o
 				}
@@ -944,8 +1151,7 @@ func compare(a, b interface{}) ordering {
 				return equal
 			case a.Len() < b.Len():
 				return less
-			default:
-				// a.Len() > b.Len()
+			default: // a.Len() > b.Len()
 				return more
 			}
 		}
