@@ -2,9 +2,10 @@ package eval
 
 import (
 	"sync"
+	"sync/atomic"
 
-	"github.com/elves/elvish/pkg/diag"
-	"github.com/elves/elvish/pkg/eval/vals"
+	"src.elv.sh/pkg/diag"
+	"src.elv.sh/pkg/eval/vals"
 )
 
 // Flow control.
@@ -41,22 +42,29 @@ func init() {
 // except that it does not perform any redirections.
 //
 // Here is an example that lets you pipe the stdout and stderr of a command to two
-// different commands:
+// different commands in order to independently capture the output of each byte stream:
 //
-// ```elvish
-// pout = (pipe)
-// perr = (pipe)
-// run-parallel {
-// foo > $pout 2> $perr
-// pwclose $pout
-// pwclose $perr
-// } {
-// bar < $pout
-// prclose $pout
-// } {
-// bar2 < $perr
-// prclose $perr
-// }
+// ```elvish-transcript
+// ~> fn capture {|f|
+//      var pout = (file:pipe)
+//      var perr = (file:pipe)
+//      var out err
+//      run-parallel {
+//        $f > $pout[w] 2> $perr[w]
+//        file:close $pout[w]
+//        file:close $perr[w]
+//      } {
+//        set out = (slurp < $pout[r])
+//        file:close $pout[r]
+//      } {
+//        set err = (slurp < $perr[r])
+//        file:close $perr[r]
+//      }
+//      put $out $err
+//    }
+// ~> capture { echo stdout-test; echo stderr-test >&2 }
+// ▶ "stdout-test\n"
+// ▶ "stderr-test\n"
 // ```
 //
 // This command is intended for doing a fixed number of heterogeneous things in
@@ -66,8 +74,8 @@ func init() {
 // @cf peach
 
 func runParallel(fm *Frame, functions ...Callable) error {
-	var waitg sync.WaitGroup
-	waitg.Add(len(functions))
+	var wg sync.WaitGroup
+	wg.Add(len(functions))
 	exceptions := make([]Exception, len(functions))
 	for i, function := range functions {
 		go func(fm2 *Frame, function Callable, pexc *Exception) {
@@ -75,11 +83,11 @@ func runParallel(fm *Frame, functions ...Callable) error {
 			if err != nil {
 				*pexc = err.(Exception)
 			}
-			waitg.Done()
+			wg.Done()
 		}(fm.fork("[run-parallel function]"), function, &exceptions[i])
 	}
 
-	waitg.Wait()
+	wg.Wait()
 	return MakePipelineError(exceptions)
 }
 
@@ -89,14 +97,22 @@ func runParallel(fm *Frame, functions ...Callable) error {
 // each $f $input-list?
 // ```
 //
-// Call `$f` on all inputs. Examples:
+// Call `$f` on all inputs.
+//
+// An exception raised from [`break`](#break) is caught by `each`, and will
+// cause it to terminate early.
+//
+// An exception raised from [`continue`](#continue) is swallowed and can be used
+// to terminate a single iteration early.
+//
+// Examples:
 //
 // ```elvish-transcript
-// ~> range 5 8 | each [x]{ ^ $x 2 }
+// ~> range 5 8 | each {|x| * $x $x }
 // ▶ 25
 // ▶ 36
 // ▶ 49
-// ~> each [x]{ put $x[:3] } [lorem ipsum]
+// ~> each {|x| put $x[:3] } [lorem ipsum]
 // ▶ lor
 // ▶ ips
 // ```
@@ -139,18 +155,32 @@ func each(fm *Frame, f Callable, inputs Inputs) error {
 // peach $f $input-list?
 // ```
 //
-// Call `$f` on all inputs, possibly in parallel.
+// Calls `$f` on all inputs, possibly in parallel.
+//
+// Like `each`, an exception raised from [`break`](#break) will cause `peach`
+// to terminate early. However due to the parallel nature of `peach`, the exact
+// time of termination is non-deterministic and not even guaranteed.
+//
+// An exception raised from [`continue`](#continue) is swallowed and can be used
+// to terminate a single iteration early.
 //
 // Example (your output will differ):
 //
 // ```elvish-transcript
-// ~> range 1 7 | peach [x]{ + $x 10 }
-// ▶ 12
-// ▶ 11
-// ▶ 13
-// ▶ 16
-// ▶ 15
-// ▶ 14
+// ~> range 1 10 | peach {|x| + $x 10 }
+// ▶ (num 12)
+// ▶ (num 13)
+// ▶ (num 11)
+// ▶ (num 16)
+// ▶ (num 18)
+// ▶ (num 14)
+// ▶ (num 17)
+// ▶ (num 15)
+// ▶ (num 19)
+// ~> range 1 101 |
+//    peach {|x| if (== 50 $x) { break } else { put $x } } |
+//    + (all) # 1+...+49 = 1225; 1+...+100 = 5050
+// ▶ (num 1328)
 // ```
 //
 // This command is intended for homogeneous processing of possibly unbound data. If
@@ -160,14 +190,16 @@ func each(fm *Frame, f Callable, inputs Inputs) error {
 // @cf each run-parallel
 
 func peach(fm *Frame, f Callable, inputs Inputs) error {
-	var w sync.WaitGroup
-	broken := false
+	var wg sync.WaitGroup
+	var broken int32
+	var errMu sync.Mutex
 	var err error
+
 	inputs(func(v interface{}) {
-		if broken || err != nil {
+		if atomic.LoadInt32(&broken) != 0 {
 			return
 		}
-		w.Add(1)
+		wg.Add(1)
 		go func() {
 			newFm := fm.fork("closure of peach")
 			newFm.ports[0] = DummyInputPort
@@ -179,16 +211,18 @@ func peach(fm *Frame, f Callable, inputs Inputs) error {
 				case nil, Continue:
 					// nop
 				case Break:
-					broken = true
+					atomic.StoreInt32(&broken, 1)
 				default:
-					broken = true
+					errMu.Lock()
 					err = diag.Errors(err, ex)
+					defer errMu.Unlock()
+					atomic.StoreInt32(&broken, 1)
 				}
 			}
-			w.Done()
+			wg.Done()
 		}()
 	})
-	w.Wait()
+	wg.Wait()
 	return err
 }
 
@@ -249,14 +283,14 @@ func multiErrorFn(excs ...Exception) error {
 //elvdoc:fn return
 //
 // Raises the special "return" exception. When raised inside a named function
-// (defined by the [`fn` keyword](../language.html#function-definition-fn)) it
-// is captured by the function and causes the function to terminate. It is not
-// captured by an anonymous function (aka [lambda](../language.html#lambda)).
+// (defined by the [`fn` keyword](language.html#fn)) it is captured by the
+// function and causes the function to terminate. It is not captured by an
+// ordinary anonymous function.
 //
 // Because `return` raises an exception it can be caught by a
-// [`try`](language.html#exception-control-try) block. If not caught, either
-// implicitly by a named function or explicitly, it causes a failure like any
-// other uncaught exception.
+// [`try`](language.html#try) block. If not caught, either implicitly by a
+// named function or explicitly, it causes a failure like any other uncaught
+// exception.
 //
 // See the discussion about [flow commands and
 // exceptions](language.html#exception-and-flow-commands)
@@ -266,6 +300,7 @@ func multiErrorFn(excs ...Exception) error {
 // raised by return. Consider this example:
 //
 // ```elvish-transcript
+// ~> use builtin
 // ~> fn return { put return; builtin:return }
 // ~> fn test-return { put before; return; put after }
 // ~> test-return
@@ -274,10 +309,11 @@ func multiErrorFn(excs ...Exception) error {
 // ▶ after
 // ```
 //
-// Instead, shadow the function by directly assigning to `local:return~`:
+// Instead, shadow the function by directly assigning to `return~`:
 //
 // ```elvish-transcript
-// ~> local:return~ = { put return; builtin:return }
+// ~> use builtin
+// ~> var return~ = { put return; builtin:return }
 // ~> fn test-return { put before; return; put after }
 // ~> test-return
 // ▶ before
@@ -294,9 +330,8 @@ func returnFn() error {
 // captured and causes the loop to terminate.
 //
 // Because `break` raises an exception it can be caught by a
-// [`try`](language.html#exception-control-try) block. If not caught, either
-// implicitly by a loop or explicitly, it causes a failure like any other
-// uncaught exception.
+// [`try`](language.html#try) block. If not caught, either implicitly by a loop
+// or explicitly, it causes a failure like any other uncaught exception.
 //
 // See the discussion about [flow commands and exceptions](language.html#exception-and-flow-commands)
 //
@@ -304,8 +339,9 @@ func returnFn() error {
 // command. If you do so you should explicitly invoke the builtin. For example:
 //
 // ```elvish-transcript
-// > fn break []{ put 'break'; builtin:break; put 'should not appear' }
-// > for x [a b c] { put $x; break; put 'unexpected' }
+// ~> use builtin
+// ~> fn break { put 'break'; builtin:break; put 'should not appear' }
+// ~> for x [a b c] { put $x; break; put 'unexpected' }
 // ▶ a
 // ▶ break
 // ```
@@ -320,9 +356,8 @@ func breakFn() error {
 // captured and causes the loop to begin its next iteration.
 //
 // Because `continue` raises an exception it can be caught by a
-// [`try`](language.html#exception-control-try) block. If not caught, either
-// implicitly by a loop or explicitly, it causes a failure like any other
-// uncaught exception.
+// [`try`](language.html#try) block. If not caught, either implicitly by a loop
+// or explicitly, it causes a failure like any other uncaught exception.
 //
 // See the discussion about [flow commands and exceptions](language.html#exception-and-flow-commands)
 //
@@ -330,8 +365,9 @@ func breakFn() error {
 // command. If you do so you should explicitly invoke the builtin. For example:
 //
 // ```elvish-transcript
-// > fn break []{ put 'continue'; builtin:continue; put 'should not appear' }
-// > for x [a b c] { put $x; continue; put 'unexpected' }
+// ~> use builtin
+// ~> fn continue { put 'continue'; builtin:continue; put 'should not appear' }
+// ~> for x [a b c] { put $x; continue; put 'unexpected' }
 // ▶ a
 // ▶ continue
 // ▶ b
