@@ -2,11 +2,11 @@ package eval
 
 import (
 	"errors"
-	"os"
-	"path/filepath"
 
-	"github.com/elves/elvish/pkg/fsutil"
-	"github.com/elves/elvish/pkg/store"
+	"src.elv.sh/pkg/fsutil"
+	"src.elv.sh/pkg/store/storedefs"
+
+	"src.elv.sh/pkg/eval/errs"
 )
 
 // Filesystem commands.
@@ -14,22 +14,6 @@ import (
 // ErrStoreNotConnected is thrown by dir-history when the store is not connected.
 var ErrStoreNotConnected = errors.New("store not connected")
 
-//elvdoc:fn path-\*
-//
-// ```elvish
-// path-abs $path
-// path-base $path
-// path-clean $path
-// path-dir $path
-// path-ext $path
-// ```
-//
-// See [godoc of path/filepath](https://godoc.org/path/filepath). Go errors are
-// turned into exceptions.
-//
-// These functions are deprecated. Use the equivalent functions in the
-// [path:](path.html) module.
-
 func init() {
 	addBuiltinFns(map[string]interface{}{
 		// Directory
@@ -37,16 +21,7 @@ func init() {
 		"dir-history": dirs,
 
 		// Path
-		"path-abs":      filepath.Abs,
-		"path-base":     filepath.Base,
-		"path-clean":    filepath.Clean,
-		"path-dir":      filepath.Dir,
-		"path-ext":      filepath.Ext,
-		"eval-symlinks": filepath.EvalSymlinks,
-		"tilde-abbr":    tildeAbbr,
-
-		// File types
-		"-is-dir": isDir,
+		"tilde-abbr": tildeAbbr,
 	})
 }
 
@@ -76,7 +51,7 @@ func cd(fm *Frame, args ...string) error {
 	case 1:
 		dir = args[0]
 	default:
-		return ErrArgs
+		return errs.ArityMismatch{What: "arguments", ValidLow: 0, ValidHigh: 1, Actual: len(args)}
 	}
 
 	return fm.Evaler.Chdir(dir)
@@ -88,8 +63,8 @@ func cd(fm *Frame, args ...string) error {
 // dir-history
 // ```
 //
-// Return a list containing the directory history. Each element is a map with two
-// keys: `path` and `score`. The list is sorted by descending score.
+// Return a list containing the interactive directory history. Each element is a map with two keys:
+// `path` and `score`. The list is sorted by descending score.
 //
 // Example:
 //
@@ -97,6 +72,8 @@ func cd(fm *Frame, args ...string) error {
 // ~> dir-history | take 1
 // ▶ [&path=/Users/foo/.elvish &score=96.79928]
 // ```
+//
+// @cf edit:command-history
 
 type dirHistoryEntry struct {
 	Path  string
@@ -106,17 +83,20 @@ type dirHistoryEntry struct {
 func (dirHistoryEntry) IsStructMap() {}
 
 func dirs(fm *Frame) error {
-	daemon := fm.Evaler.DaemonClient()
+	daemon := fm.Evaler.DaemonClient
 	if daemon == nil {
 		return ErrStoreNotConnected
 	}
-	dirs, err := daemon.Dirs(store.NoBlacklist)
+	dirs, err := daemon.Dirs(storedefs.NoBlacklist)
 	if err != nil {
 		return err
 	}
-	out := fm.OutputChan()
+	out := fm.ValueOutput()
 	for _, dir := range dirs {
-		out <- dirHistoryEntry{dir.Path, dir.Score}
+		err := out.Put(dirHistoryEntry{dir.Path, dir.Score})
+		if err != nil {
+			return err
+		}
 	}
 	return nil
 }
@@ -144,8 +124,3 @@ func dirs(fm *Frame) error {
 func tildeAbbr(path string) string {
 	return fsutil.TildeAbbr(path)
 }
-
-func isDir(path string) bool {
-	fi, err := os.Stat(path)
-	return err == nil && fi.Mode().IsDir()
-}
