@@ -3,7 +3,8 @@ package eval_test
 import (
 	"testing"
 
-	. "github.com/elves/elvish/pkg/eval/evaltest"
+	. "src.elv.sh/pkg/eval"
+	. "src.elv.sh/pkg/eval/evaltest"
 )
 
 func TestStringComparisonCommands(t *testing.T) {
@@ -27,22 +28,8 @@ func TestStringComparisonCommands(t *testing.T) {
 
 func TestToString(t *testing.T) {
 	Test(t,
-		That(`to-string str (float64 1) $true`).Puts("str", "1", "$true"),
-	)
-}
-
-func TestOrd(t *testing.T) {
-	Test(t,
-		That(`ord a`).Puts("0x61"),
-		That(`ord 你好`).Puts("0x4f60", "0x597d"),
-	)
-}
-
-func TestChr(t *testing.T) {
-	Test(t,
-		That(`chr 0x61`).Puts("a"),
-		That(`chr 0x4f60 0x597d`).Puts("你好"),
-		That(`chr -1`).Throws(AnyError),
+		That(`to-string str (num 1) $true`).Puts("str", "1", "$true"),
+		thatOutputErrorIsBubbled("to-string str"),
 	)
 }
 
@@ -52,33 +39,53 @@ func TestBase(t *testing.T) {
 		That(`base 16 42 233`).Puts("2a", "e9"),
 		That(`base 1 1`).Throws(AnyError),   // no base-1
 		That(`base 37 10`).Throws(AnyError), // no letter for base-37
+		thatOutputErrorIsBubbled("base 2 1"),
 	)
 }
 
 func TestWcswidth(t *testing.T) {
 	Test(t,
-		That(`wcswidth 你好`).Puts("4"),
+		That(`wcswidth 你好`).Puts(4),
 		That(`-override-wcwidth x 10; wcswidth 1x2x; -override-wcwidth x 1`).
-			Puts("22"),
-	)
-}
-
-func TestHasPrefix(t *testing.T) {
-	Test(t,
-		That(`has-prefix golang go`).Puts(true),
-		That(`has-prefix golang x`).Puts(false),
-	)
-}
-
-func TestHasSuffix(t *testing.T) {
-	Test(t,
-		That(`has-suffix golang x`).Puts(false),
+			Puts(22),
 	)
 }
 
 func TestEawk(t *testing.T) {
 	Test(t,
-		That(`echo "  ax  by cz  \n11\t22 33" | eawk [@a]{ put $a[-1] }`).
+		That(`echo "  ax  by cz  \n11\t22 33" | eawk {|@a| put $a[-1] }`).
 			Puts("cz", "33"),
+		// Bad input type
+		That(`num 42 | eawk {|@a| fail "this should not run" }`).
+			Throws(ErrInputOfEawkMustBeString),
+		// Propagation of exception
+		That(`
+			to-lines [1 2 3 4] | eawk {|@a|
+				if (==s 3 $a[1]) {
+					fail "stop eawk"
+				}
+				put $a[1]
+			}
+		`).Puts("1", "2").Throws(FailError{"stop eawk"}),
+		// break
+		That(`
+			to-lines [" a" "b\tc " "d" "e"] | eawk {|@a|
+				if (==s d $a[1]) {
+					break
+				} else {
+					put $a[-1]
+				}
+			}
+		`).Puts("a", "c"),
+		// continue
+		That(`
+			to-lines [" a" "b\tc " "d" "e"] | eawk {|@a|
+				if (==s d $a[1]) {
+					continue
+				} else {
+					put $a[-1]
+				}
+			}
+		`).Puts("a", "c", "e"),
 	)
 }
