@@ -22,7 +22,6 @@ import (
 	"strings"
 
 	"src.elv.sh/pkg/diag"
-	"src.elv.sh/pkg/eval/mods/bundled"
 	"src.elv.sh/pkg/eval/vals"
 	"src.elv.sh/pkg/eval/vars"
 	"src.elv.sh/pkg/parse"
@@ -37,9 +36,11 @@ var builtinSpecials map[string]compileBuiltin
 // intended for external consumption, e.g. the syntax highlighter.
 var IsBuiltinSpecial = map[string]bool{}
 
-type noSuchModule struct{ spec string }
+// NoSuchModule encodes an error where a module spec cannot be resolved.
+type NoSuchModule struct{ spec string }
 
-func (err noSuchModule) Error() string { return "no such module: " + err.spec }
+// Error implements the error interface.
+func (err NoSuchModule) Error() string { return "no such module: " + err.spec }
 
 func init() {
 	// Needed to avoid initialization loop
@@ -51,13 +52,16 @@ func init() {
 
 		"use": compileUse,
 
-		"and": compileAnd,
-		"or":  compileOr,
+		"and":      compileAnd,
+		"or":       compileOr,
+		"coalesce": compileCoalesce,
 
 		"if":    compileIf,
 		"while": compileWhile,
 		"for":   compileFor,
 		"try":   compileTry,
+
+		"pragma": compilePragma,
 	}
 	for name := range builtinSpecials {
 		IsBuiltinSpecial[name] = true
@@ -66,47 +70,30 @@ func init() {
 
 // VarForm = 'var' { VariablePrimary } [ '=' { Compound } ]
 func compileVar(cp *compiler, fn *parse.Form) effectOp {
-	lhs := lvaluesGroup{rest: -1}
+	eqIndex := -1
 	for i, cn := range fn.Args {
 		if parse.SourceText(cn) == "=" {
-			var rhs valuesOp
-			if i == len(fn.Args)-1 {
-				rhs = nopValuesOp{diag.PointRanging(fn.Range().To)}
-			} else {
-				rhs = seqValuesOp{
-					diag.MixedRanging(fn.Args[i+1], fn.Args[len(fn.Args)-1]),
-					cp.compoundOps(fn.Args[i+1:])}
-			}
-			return &assignOp{fn.Range(), lhs, rhs}
+			eqIndex = i
+			break
 		}
-		if len(cn.Indexings) != 1 {
-			cp.errorpf(cn, "variable name must be a single string literal")
-		}
-		if len(cn.Indexings[0].Indicies) > 0 {
-			cp.errorpf(cn, "variable name must not have indicies")
-		}
-		pn := cn.Indexings[0].Head
-		if !parse.ValidLHSVariable(pn, true) {
-			cp.errorpf(cn, "invalid variable name")
-		}
-
-		name := pn.Value
-		if !IsUnqualified(name) {
-			cp.errorpf(cn, "variable declared in var must be unqualified")
-		}
-		sigil, name := SplitSigil(name)
-		if sigil == "@" {
-			if lhs.rest != -1 {
-				cp.errorpf(cn, "multiple variable names with @ not allowed")
-			}
-			lhs.rest = i
-		}
-		slotIndex := cp.thisScope().add(name)
-		lhs.lvalues = append(lhs.lvalues,
-			lvalue{cn.Range(), &varRef{localScope, slotIndex, nil}, nil, nil})
 	}
-	// If there is no assignment, there is no work to be done at eval-time.
-	return nopOp{}
+
+	if eqIndex == -1 {
+		cp.parseCompoundLValues(fn.Args, newLValue)
+		// Just create new variables, nothing extra to do at runtime.
+		return nopOp{}
+	}
+	// Compile rhs before lhs before many potential shadowing
+	var rhs valuesOp
+	if eqIndex == len(fn.Args)-1 {
+		rhs = nopValuesOp{diag.PointRanging(fn.Range().To)}
+	} else {
+		rhs = seqValuesOp{
+			diag.MixedRanging(fn.Args[eqIndex+1], fn.Args[len(fn.Args)-1]),
+			cp.compoundOps(fn.Args[eqIndex+1:])}
+	}
+	lhs := cp.parseCompoundLValues(fn.Args[:eqIndex], newLValue)
+	return &assignOp{fn.Range(), lhs, rhs}
 }
 
 // IsUnqualified returns whether name is an unqualified variable name.
@@ -127,7 +114,7 @@ func compileSet(cp *compiler, fn *parse.Form) effectOp {
 	if eq == -1 {
 		cp.errorpf(diag.PointRanging(fn.Range().To), "need = and right-hand-side")
 	}
-	lhs := cp.parseCompoundLValues(fn.Args[:eq])
+	lhs := cp.parseCompoundLValues(fn.Args[:eq], setLValue)
 	var rhs valuesOp
 	if eq == len(fn.Args)-1 {
 		rhs = nopValuesOp{diag.PointRanging(fn.Range().To)}
@@ -150,7 +137,7 @@ func compileDel(cp *compiler, fn *parse.Form) effectOp {
 			cp.errorpf(cn, delArgMsg)
 			continue
 		}
-		head, indices := cn.Indexings[0].Head, cn.Indexings[0].Indicies
+		head, indices := cn.Indexings[0].Head, cn.Indexings[0].Indices
 		if head.Type == parse.Variable {
 			cp.errorpf(cn, "arguments to del must drop $")
 		} else if !parse.ValidLHSVariable(head, false) {
@@ -169,7 +156,7 @@ func compileDel(cp *compiler, fn *parse.Form) effectOp {
 				f = delEnvVarOp{fn.Range(), ref.subNames[0]}
 			} else if ref.scope == localScope && len(ref.subNames) == 0 {
 				f = delLocalVarOp{ref.index}
-				cp.thisScope().deleted[ref.index] = true
+				cp.thisScope().infos[ref.index].deleted = true
 			} else {
 				cp.errorpf(cn, "only variables in local: or E: can be deleted")
 				continue
@@ -242,7 +229,7 @@ func (op *delElemOp) exec(fm *Frame) Exception {
 
 // FnForm = 'fn' StringPrimary LambdaPrimary
 //
-// fn f []{foobar} is a shorthand for set '&'f = []{foobar}.
+// fn f { foobar } is a shorthand for set '&'f = { foobar }.
 func compileFn(cp *compiler, fn *parse.Form) effectOp {
 	args := cp.walkArgs(fn)
 	nameNode := args.next()
@@ -331,9 +318,12 @@ func (op useOp) exec(fm *Frame) Exception {
 	return nil
 }
 
-var bundledModules = bundled.Get()
-
+// TODO: Add support for module specs relative to a package/workspace.
+// See https://github.com/elves/elvish/issues/1421.
 func use(fm *Frame, spec string, r diag.Ranger) (*Ns, error) {
+	// Handle relative imports. Note that this deliberately does not support Windows backslash as a
+	// path separator because module specs are meant to be platform independent. If necessary, we
+	// translate a module spec to an appropriate path for the platform.
 	if strings.HasPrefix(spec, "./") || strings.HasPrefix(spec, "../") {
 		var dir string
 		if fm.srcMeta.IsFile {
@@ -348,23 +338,30 @@ func use(fm *Frame, spec string, r diag.Ranger) (*Ns, error) {
 		path := filepath.Clean(dir + "/" + spec)
 		return useFromFile(fm, spec, path, r)
 	}
+
+	// Handle imports of pre-defined modules like `builtin` and `str`.
 	if ns, ok := fm.Evaler.modules[spec]; ok {
 		return ns, nil
 	}
-	if code, ok := bundledModules[spec]; ok {
+	if code, ok := fm.Evaler.BundledModules[spec]; ok {
 		return evalModule(fm, spec,
 			parse.Source{Name: "[bundled " + spec + "]", Code: code}, r)
 	}
+
+	// Handle imports relative to the Elvish module search directories.
+	//
 	// TODO: For non-relative imports, use the spec (instead of the full path)
 	// as the module key instead to avoid searching every time.
-	for _, dir := range fm.Evaler.getLibDirs() {
+	for _, dir := range fm.Evaler.LibDirs {
 		ns, err := useFromFile(fm, spec, filepath.Join(dir, spec), r)
-		if _, isNoSuchModule := err.(noSuchModule); isNoSuchModule {
+		if _, isNoSuchModule := err.(NoSuchModule); isNoSuchModule {
 			continue
 		}
 		return ns, err
 	}
-	return nil, noSuchModule{spec}
+
+	// Sadly, we couldn't resolve the module spec.
+	return nil, NoSuchModule{spec}
 }
 
 // TODO: Make access to fm.Evaler.modules concurrency-safe.
@@ -377,7 +374,7 @@ func useFromFile(fm *Frame, spec, path string, r diag.Ranger) (*Ns, error) {
 		code, err := readFileUTF8(path + ".elv")
 		if err != nil {
 			if os.IsNotExist(err) {
-				return nil, noSuchModule{spec}
+				return nil, NoSuchModule{spec}
 			}
 			return nil, err
 		}
@@ -387,7 +384,7 @@ func useFromFile(fm *Frame, spec, path string, r diag.Ranger) (*Ns, error) {
 
 	plug, err := pluginOpen(path + ".so")
 	if err != nil {
-		return nil, noSuchModule{spec}
+		return nil, NoSuchModule{spec}
 	}
 	sym, err := plug.Lookup("Ns")
 	if err != nil {
@@ -395,7 +392,7 @@ func useFromFile(fm *Frame, spec, path string, r diag.Ranger) (*Ns, error) {
 	}
 	ns, ok := sym.(**Ns)
 	if !ok {
-		return nil, noSuchModule{spec}
+		return nil, NoSuchModule{spec}
 	}
 	fm.Evaler.modules[path] = *ns
 	return *ns, nil
@@ -464,6 +461,33 @@ func (op *andOrOp) exec(fm *Frame) Exception {
 	return fm.errorp(op, out.Put(lastValue))
 }
 
+// Compiles the "coalesce" special form, which is like "or", but evaluates until
+// a non-nil value is found.
+func compileCoalesce(cp *compiler, fn *parse.Form) effectOp {
+	return &coalesceOp{fn.Range(), cp.compoundOps(fn.Args)}
+}
+
+type coalesceOp struct {
+	diag.Ranging
+	argOps []valuesOp
+}
+
+func (op *coalesceOp) exec(fm *Frame) Exception {
+	out := fm.ValueOutput()
+	for _, argOp := range op.argOps {
+		values, exc := argOp.exec(fm)
+		if exc != nil {
+			return exc
+		}
+		for _, value := range values {
+			if value != nil {
+				return fm.errorp(op, out.Put(value))
+			}
+		}
+	}
+	return fm.errorp(op, out.Put(nil))
+}
+
 func compileIf(cp *compiler, fn *parse.Form) effectOp {
 	args := cp.walkArgs(fn)
 	var condNodes []*parse.Compound
@@ -581,7 +605,7 @@ func compileFor(cp *compiler, fn *parse.Form) effectOp {
 	elseNode := args.nextMustLambdaIfAfter("else")
 	args.mustEnd()
 
-	lvalue := cp.compileOneLValue(varNode)
+	lvalue := cp.compileOneLValue(varNode, setLValue|newLValue)
 
 	iterOp := cp.compoundOp(iterNode)
 	bodyOp := cp.primaryOp(bodyNode)
@@ -675,7 +699,7 @@ func compileTry(cp *compiler, fn *parse.Form) effectOp {
 	var bodyOp, exceptOp, elseOp, finallyOp valuesOp
 	bodyOp = cp.primaryOp(bodyNode)
 	if exceptVarNode != nil {
-		exceptVar = cp.compileOneLValue(exceptVarNode)
+		exceptVar = cp.compileOneLValue(exceptVarNode, setLValue|newLValue)
 	}
 	if exceptNode != nil {
 		exceptOp = cp.primaryOp(exceptNode)
@@ -740,11 +764,42 @@ func (op *tryOp) exec(fm *Frame) Exception {
 	return fm.errorp(op, err)
 }
 
-func (cp *compiler) compileOneLValue(n *parse.Compound) lvalue {
+// PragmaForm = 'pragma' 'fallback-resolver' '=' { Compound }
+func compilePragma(cp *compiler, fn *parse.Form) effectOp {
+	args := cp.walkArgs(fn)
+	nameNode := args.next()
+	name := stringLiteralOrError(cp, nameNode, "pragma name")
+	eqNode := args.next()
+	eq := stringLiteralOrError(cp, eqNode, "literal =")
+	if eq != "=" {
+		cp.errorpf(eqNode, "must be literal =")
+	}
+	valueNode := args.next()
+	args.mustEnd()
+
+	switch name {
+	case "unknown-command":
+		value := stringLiteralOrError(cp, valueNode, "value for unknown-command")
+		switch value {
+		case "disallow":
+			cp.currentPragma().unknownCommandIsExternal = false
+		case "external":
+			cp.currentPragma().unknownCommandIsExternal = true
+		default:
+			cp.errorpf(valueNode,
+				"invalid value for unknown-command: %s", parse.Quote(value))
+		}
+	default:
+		cp.errorpf(nameNode, "unknown pragma %s", parse.Quote(name))
+	}
+	return nopOp{}
+}
+
+func (cp *compiler) compileOneLValue(n *parse.Compound, f lvalueFlag) lvalue {
 	if len(n.Indexings) != 1 {
 		cp.errorpf(n, "must be valid lvalue")
 	}
-	lvalues := cp.parseIndexingLValue(n.Indexings[0])
+	lvalues := cp.parseIndexingLValue(n.Indexings[0], f)
 	if lvalues.rest != -1 {
 		cp.errorpf(lvalues.lvalues[lvalues.rest], "rest variable not allowed")
 	}
