@@ -7,12 +7,12 @@ import (
 	"strings"
 	"unsafe"
 
-	"github.com/elves/elvish/pkg/diag"
-	"github.com/elves/elvish/pkg/eval/errs"
-	"github.com/elves/elvish/pkg/eval/vals"
-	"github.com/elves/elvish/pkg/eval/vars"
-	"github.com/elves/elvish/pkg/parse"
-	"github.com/xiaq/persistent/hash"
+	"src.elv.sh/pkg/diag"
+	"src.elv.sh/pkg/eval/errs"
+	"src.elv.sh/pkg/eval/vals"
+	"src.elv.sh/pkg/eval/vars"
+	"src.elv.sh/pkg/parse"
+	"src.elv.sh/pkg/persistent/hash"
 )
 
 // A user-defined function in Elvish code. Each closure has its unique identity.
@@ -23,7 +23,7 @@ type closure struct {
 	OptNames    []string
 	OptDefaults []interface{}
 	Op          effectOp
-	NewLocal    []string
+	NewLocal    []staticVarInfo
 	Captured    *Ns
 	SrcMeta     parse.Source
 	DefRange    diag.Ranging
@@ -56,14 +56,12 @@ func (c *closure) Call(fm *Frame, args []interface{}, opts map[string]interface{
 	// Check number of arguments.
 	if c.RestArg != -1 {
 		if len(args) < len(c.ArgNames)-1 {
-			return errs.ArityMismatch{
-				What:     "arguments here",
+			return errs.ArityMismatch{What: "arguments",
 				ValidLow: len(c.ArgNames) - 1, ValidHigh: -1, Actual: len(args)}
 		}
 	} else {
 		if len(args) != len(c.ArgNames) {
-			return errs.ArityMismatch{
-				What:     "arguments here",
+			return errs.ArityMismatch{What: "arguments",
 				ValidLow: len(c.ArgNames), ValidHigh: len(c.ArgNames), Actual: len(args)}
 		}
 	}
@@ -99,13 +97,13 @@ func (c *closure) Call(fm *Frame, args []interface{}, opts map[string]interface{
 
 	// Populate local scope with arguments, options, and newly created locals.
 	localSize := len(c.ArgNames) + len(c.OptNames) + len(c.NewLocal)
-	local := &Ns{make([]vars.Var, localSize), make([]string, localSize), make([]bool, localSize)}
+	local := &Ns{make([]vars.Var, localSize), make([]staticVarInfo, localSize)}
 
 	for i, name := range c.ArgNames {
-		local.names[i] = name
+		local.infos[i] = staticVarInfo{name, false, false}
 	}
 	if c.RestArg == -1 {
-		for i, _ := range c.ArgNames {
+		for i := range c.ArgNames {
 			local.slots[i] = vars.FromInit(args[i])
 		}
 	} else {
@@ -126,14 +124,15 @@ func (c *closure) Call(fm *Frame, args []interface{}, opts map[string]interface{
 		if !ok {
 			v = c.OptDefaults[i]
 		}
-		local.names[offset+i] = name
+		local.infos[offset+i] = staticVarInfo{name, false, false}
 		local.slots[offset+i] = vars.FromInit(v)
 	}
 
 	offset += len(c.OptNames)
-	for i, name := range c.NewLocal {
-		local.names[offset+i] = name
-		local.slots[offset+i] = MakeVarFromName(name)
+	for i, info := range c.NewLocal {
+		local.infos[offset+i] = info
+		// TODO: Take info.readOnly into account too when creating variable
+		local.slots[offset+i] = MakeVarFromName(info.name)
 	}
 
 	fm.local = local
@@ -141,15 +140,20 @@ func (c *closure) Call(fm *Frame, args []interface{}, opts map[string]interface{
 	return c.Op.exec(fm)
 }
 
+var (
+	fnDefault = NewGoFn("nop~", nop)
+	nsDefault = &Ns{}
+)
+
 // MakeVarFromName creates a Var with a suitable type constraint inferred from
 // the name.
 func MakeVarFromName(name string) vars.Var {
 	switch {
 	case strings.HasSuffix(name, FnSuffix):
-		val := Callable(nil)
+		val := fnDefault
 		return vars.FromPtr(&val)
 	case strings.HasSuffix(name, NsSuffix):
-		val := (*Ns)(nil)
+		val := nsDefault
 		return vars.FromPtr(&val)
 	default:
 		return vars.FromInit(nil)
