@@ -4,13 +4,15 @@ import (
 	"fmt"
 	"os"
 	"sync"
+	"sync/atomic"
 
-	"github.com/elves/elvish/pkg/diag"
-	"github.com/elves/elvish/pkg/eval/errs"
-	"github.com/elves/elvish/pkg/eval/vals"
-	"github.com/elves/elvish/pkg/eval/vars"
-	"github.com/elves/elvish/pkg/fsutil"
-	"github.com/elves/elvish/pkg/parse"
+	"src.elv.sh/pkg/diag"
+	"src.elv.sh/pkg/eval/errs"
+	"src.elv.sh/pkg/eval/vals"
+	"src.elv.sh/pkg/eval/vars"
+	"src.elv.sh/pkg/fsutil"
+	"src.elv.sh/pkg/parse"
+	"src.elv.sh/pkg/parse/cmpd"
 )
 
 // An operation with some side effects.
@@ -74,11 +76,6 @@ func (op *pipelineOp) exec(fm *Frame) Exception {
 		fm.intCh = nil
 		fm.background = true
 		fm.Evaler.addNumBgJobs(1)
-
-		if fm.Evaler.Editor() != nil {
-			// TODO: Redirect output in interactive mode so that the line
-			// editor does not get messed up.
-		}
 	}
 
 	nforms := len(op.subops)
@@ -91,12 +88,13 @@ func (op *pipelineOp) exec(fm *Frame) Exception {
 
 	// For each form, create a dedicated evalCtx and run asynchronously
 	for i, formOp := range op.subops {
-		hasChanInput := i > 0
 		newFm := fm.fork("[form op]")
-		if i > 0 {
+		inputIsPipe := i > 0
+		outputIsPipe := i < nforms-1
+		if inputIsPipe {
 			newFm.ports[0] = nextIn
 		}
-		if i < nforms-1 {
+		if outputIsPipe {
 			// Each internal port pair consists of a (byte) pipe pair and a
 			// channel.
 			// os.Pipe sets O_CLOEXEC, which is what we want.
@@ -105,29 +103,38 @@ func (op *pipelineOp) exec(fm *Frame) Exception {
 				return fm.errorpf(op, "failed to create pipe: %s", e)
 			}
 			ch := make(chan interface{}, pipelineChanBufferSize)
+			sendStop := make(chan struct{})
+			sendError := new(error)
+			readerGone := new(int32)
 			newFm.ports[1] = &Port{
-				File: writer, Chan: ch, closeFile: true, closeChan: true}
+				File: writer, Chan: ch,
+				closeFile: true, closeChan: true,
+				sendStop: sendStop, sendError: sendError, readerGone: readerGone}
 			nextIn = &Port{
-				File: reader, Chan: ch, closeFile: true, closeChan: false}
+				File: reader, Chan: ch,
+				closeFile: true, closeChan: false,
+				// Store in input port for ease of retrieval later
+				sendStop: sendStop, sendError: sendError, readerGone: readerGone}
 		}
-		thisOp := formOp
-		thisExc := &excs[i]
-		go func() {
-			exc := thisOp.exec(newFm)
+		f := func(formOp effectOp, pexc *Exception) {
+			exc := formOp.exec(newFm)
+			if exc != nil && !(outputIsPipe && isReaderGone(exc)) {
+				*pexc = exc
+			}
+			if inputIsPipe {
+				input := newFm.ports[0]
+				*input.sendError = errs.ReaderGone{}
+				close(input.sendStop)
+				atomic.StoreInt32(input.readerGone, 1)
+			}
 			newFm.Close()
-			if exc != nil {
-				*thisExc = exc
-			}
 			wg.Done()
-			if hasChanInput {
-				// If the command has channel input, drain it. This
-				// mitigates the effect of erroneous pipelines like
-				// "range 100 | cat"; without draining the pipeline will
-				// lock up.
-				for range newFm.InputChan() {
-				}
-			}
-		}()
+		}
+		if i == nforms-1 && !op.bg {
+			f(formOp, &excs[i])
+		} else {
+			go f(formOp, &excs[i])
+		}
 	}
 
 	if op.bg {
@@ -135,17 +142,14 @@ func (op *pipelineOp) exec(fm *Frame) Exception {
 		go func() {
 			wg.Wait()
 			fm.Evaler.addNumBgJobs(-1)
-			msg := "job " + op.source + " finished"
-			err := MakePipelineError(excs)
-			if err != nil {
-				msg += ", errors = " + err.Error()
-			}
-			if fm.Evaler.getNotifyBgJobSuccess() || err != nil {
-				editor := fm.Evaler.Editor()
-				if editor != nil {
-					editor.Notify("%s", msg)
-				} else {
-					fm.ErrorFile().WriteString(msg + "\n")
+			if notify := fm.Evaler.BgJobNotify; notify != nil {
+				msg := "job " + op.source + " finished"
+				err := MakePipelineError(excs)
+				if err != nil {
+					msg += ", errors = " + err.Error()
+				}
+				if fm.Evaler.getNotifyBgJobSuccess() || err != nil {
+					notify(msg)
 				}
 			}
 		}()
@@ -155,19 +159,22 @@ func (op *pipelineOp) exec(fm *Frame) Exception {
 	return fm.errorp(op, MakePipelineError(excs))
 }
 
+func isReaderGone(exc Exception) bool {
+	_, ok := exc.Reason().(errs.ReaderGone)
+	return ok
+}
+
 func (cp *compiler) formOp(n *parse.Form) effectOp {
 	var tempLValues []lvalue
 	var assignmentOps []effectOp
 	if len(n.Assignments) > 0 {
 		assignmentOps = cp.assignmentOps(n.Assignments)
 		if n.Head == nil {
-			// Using the syntax of temporary assignment for non-temporary
-			// assignment. Still supported as of 0.15 but deprecated.
-			cp.deprecate(n, `using the syntax of temporary assignment for non-temporary assignment is deprecated; use "var" or "set" instead`, 15)
-			return seqOp{assignmentOps}
+			cp.errorpf(n, `using the syntax of temporary assignment for non-temporary assignment is no longer supported; use "var" or "set" instead`)
+			return nopOp{}
 		}
 		for _, a := range n.Assignments {
-			lvalues := cp.parseIndexingLValue(a.Left)
+			lvalues := cp.parseIndexingLValue(a.Left, setLValue|newLValue)
 			tempLValues = append(tempLValues, lvalues.lvalues...)
 		}
 		logger.Println("temporary assignment of", len(n.Assignments), "pairs")
@@ -186,7 +193,7 @@ func (cp *compiler) formBody(n *parse.Form) formBody {
 	}
 
 	// Determine if this form is a special command.
-	if head, ok := oneString(n.Head); ok {
+	if head, ok := cmpd.StringLiteral(n.Head); ok {
 		special, _ := resolveCmdHeadInternally(cp, head, n.Head)
 		if special != nil {
 			specialOp := special(cp, n)
@@ -198,10 +205,11 @@ func (cp *compiler) formBody(n *parse.Form) formBody {
 	// argument whose source is a literal "=".
 	for i, arg := range n.Args {
 		if parse.SourceText(arg) == "=" {
+			cp.deprecate(n, "legacy assignment form is deprecated, use var or set instead; migrate scripts with https://go.elv.sh/u0.17", 17)
 			lhsNodes := make([]*parse.Compound, i+1)
 			lhsNodes[0] = n.Head
 			copy(lhsNodes[1:], n.Args[:i])
-			lhs := cp.parseCompoundLValues(lhsNodes)
+			lhs := cp.parseCompoundLValues(lhsNodes, setLValue|newLValue)
 
 			rhsOps := cp.compoundOps(n.Args[i+1:])
 			var rhsRange diag.Ranging
@@ -217,13 +225,15 @@ func (cp *compiler) formBody(n *parse.Form) formBody {
 	}
 
 	var headOp valuesOp
-	if head, ok := oneString(n.Head); ok {
+	if head, ok := cmpd.StringLiteral(n.Head); ok {
 		// Head is a literal string: resolve to function or external (special
 		// commands are already handled above).
 		if _, fnRef := resolveCmdHeadInternally(cp, head, n.Head); fnRef != nil {
 			headOp = variableOp{n.Head.Range(), false, head + FnSuffix, fnRef}
-		} else {
+		} else if cp.currentPragma().unknownCommandIsExternal || fsutil.DontSearch(head) {
 			headOp = literalValues(n.Head, NewExternalCmd(head))
+		} else {
+			cp.errorpf(n.Head, "unknown command disallowed by current pragma")
 		}
 	} else {
 		// Head is not a literal string: evaluate as a normal expression.
@@ -395,7 +405,7 @@ func evalForCommand(fm *Frame, op valuesOp, what string) (Callable, error) {
 	return nil, fm.errorp(op, errs.BadValue{
 		What:   what,
 		Valid:  "callable or string containing slash",
-		Actual: vals.Kind(value)})
+		Actual: vals.Repr(value, vals.NoPretty)})
 }
 
 func allTrue(vs []interface{}) bool {
@@ -408,7 +418,7 @@ func allTrue(vs []interface{}) bool {
 }
 
 func (cp *compiler) assignmentOp(n *parse.Assignment) effectOp {
-	lhs := cp.parseIndexingLValue(n.Left)
+	lhs := cp.parseIndexingLValue(n.Left, setLValue|newLValue)
 	rhs := cp.compoundOp(n.Right)
 	return &assignOp{n.Range(), lhs, rhs}
 }
@@ -421,16 +431,6 @@ func (cp *compiler) assignmentOps(ns []*parse.Assignment) []effectOp {
 	return ops
 }
 
-func (cp *compiler) literal(n *parse.Primary, msg string) string {
-	switch n.Type {
-	case parse.Bareword, parse.SingleQuoted, parse.DoubleQuoted:
-		return n.Value
-	default:
-		cp.errorpf(n, msg)
-		return ""
-	}
-}
-
 const defaultFileRedirPerm = 0644
 
 // redir compiles a Redir into a op.
@@ -483,21 +483,6 @@ type invalidFD struct{ fd int }
 
 func (err invalidFD) Error() string { return fmt.Sprintf("invalid fd: %d", err.fd) }
 
-// Returns a suitable dummy value for the channel part of the port when
-// redirecting from or to a file, so that the read and write attempts fail
-// silently (instead of blocking or panicking).
-//
-// TODO: Instead of letting read and write attempts fail silently, consider
-// raising an exception instead.
-func chanForFileRedir(mode parse.RedirMode) chan interface{} {
-	if mode == parse.Read {
-		// ClosedChan produces no values when reading.
-		return ClosedChan
-	}
-	// BlackholeChan discards all values written to it.
-	return BlackholeChan
-}
-
 func (op *redirOp) exec(fm *Frame) Exception {
 	var dst int
 	if op.dstOp == nil {
@@ -530,7 +515,9 @@ func (op *redirOp) exec(fm *Frame) Exception {
 		switch {
 		case src == -1:
 			// close
-			fm.ports[dst] = &Port{}
+			fm.ports[dst] = &Port{
+				// Ensure that writing to value output throws an exception
+				sendStop: closedSendStop, sendError: &ErrNoValueOutput}
 		case src >= len(fm.ports) || fm.ports[src] == nil:
 			return fm.errorp(op, invalidFD{src})
 		default:
@@ -548,9 +535,9 @@ func (op *redirOp) exec(fm *Frame) Exception {
 		if err != nil {
 			return fm.errorpf(op, "failed to open file %s: %s", vals.Repr(src, vals.NoPretty), err)
 		}
-		fm.ports[dst] = &Port{File: f, closeFile: true, Chan: chanForFileRedir(op.mode)}
+		fm.ports[dst] = fileRedirPort(op.mode, f, true)
 	case vals.File:
-		fm.ports[dst] = &Port{File: src, closeFile: false, Chan: chanForFileRedir(op.mode)}
+		fm.ports[dst] = fileRedirPort(op.mode, src, false)
 	case vals.Pipe:
 		var f *os.File
 		switch op.mode {
@@ -561,7 +548,7 @@ func (op *redirOp) exec(fm *Frame) Exception {
 		default:
 			return fm.errorpf(op, "can only use < or > with pipes")
 		}
-		fm.ports[dst] = &Port{File: f, closeFile: false, Chan: chanForFileRedir(op.mode)}
+		fm.ports[dst] = fileRedirPort(op.mode, f, false)
 	default:
 		return fm.errorp(op.srcOp, errs.BadValue{
 			What:  "redirection source",
@@ -570,6 +557,24 @@ func (op *redirOp) exec(fm *Frame) Exception {
 	return nil
 }
 
+// Creates a port that only have a file component, populating the
+// channel-related fields with suitable values depending on the redirection
+// mode.
+func fileRedirPort(mode parse.RedirMode, f *os.File, closeFile bool) *Port {
+	if mode == parse.Read {
+		return &Port{
+			File: f, closeFile: closeFile,
+			// ClosedChan produces no values when reading.
+			Chan: ClosedChan,
+		}
+	}
+	return &Port{
+		File: f, closeFile: closeFile,
+		// Throws errValueOutputIsClosed when writing.
+		Chan: nil, sendStop: closedSendStop, sendError: &ErrNoValueOutput,
+	}
+}
+
 // Makes the size of *ports at least n, adding nil's if necessary.
 func growPorts(ports *[]*Port, n int) {
 	if len(*ports) >= n {
