@@ -3,12 +3,12 @@ package eval_test
 import (
 	"testing"
 
-	. "github.com/elves/elvish/pkg/eval"
-	"github.com/elves/elvish/pkg/eval/errs"
-	"github.com/elves/elvish/pkg/testutil"
+	. "src.elv.sh/pkg/eval"
+	"src.elv.sh/pkg/eval/errs"
+	. "src.elv.sh/pkg/testutil"
 
-	. "github.com/elves/elvish/pkg/eval/evaltest"
-	"github.com/elves/elvish/pkg/eval/vals"
+	. "src.elv.sh/pkg/eval/evaltest"
+	"src.elv.sh/pkg/eval/vals"
 )
 
 func TestCompound(t *testing.T) {
@@ -39,6 +39,9 @@ func TestIndexing(t *testing.T) {
 		That("put [&key=value][key]").Puts("value"),
 		That("put [&key=value][bad]").Throws(
 			vals.NoSuchKey("bad"), "[&key=value][bad]"),
+
+		That("put (fail x)[a]").Throws(FailError{"x"}, "fail x"),
+		That("put [foo][(fail x)]").Throws(FailError{"x"}, "fail x"),
 	)
 }
 
@@ -76,10 +79,8 @@ func TestStringLiteral(t *testing.T) {
 }
 
 func TestTilde(t *testing.T) {
-	home, cleanup := testutil.InTempHome()
-	testutil.MustCreateEmpty("file1")
-	testutil.MustCreateEmpty("file2")
-	defer cleanup()
+	home := InTempHome(t)
+	ApplyDir(Dir{"file1": "", "file2": ""})
 
 	Test(t,
 		// Tilde
@@ -90,10 +91,18 @@ func TestTilde(t *testing.T) {
 		That("put ~/src/").Puts(home+"/src/"),
 		// Tilde and wildcard.
 		That("put ~/*").Puts(home+"/file1", home+"/file2"),
-		// TODO(xiaq): Add regression test for #793.
+		// TODO: Add regression test for #793.
+		// TODO: Add regression test for #1246.
 	)
 }
 
+func TestWildcard(t *testing.T) {
+	Test(t,
+		That("put ***").DoesNotCompile(),
+	)
+	// More tests in glob_test.go
+}
+
 func TestOutputCapture(t *testing.T) {
 	Test(t,
 		// Output capture
@@ -130,8 +139,6 @@ func TestVariableUse(t *testing.T) {
 		That("x = outer; { local:x = inner; put $local:x }").Puts("inner"),
 		// Pseudo-namespace up: accesses upvalues.
 		That("x = outer; { local:x = inner; put $up:x }").Puts("outer"),
-		// Pseudo-namespace builtin: accesses builtins.
-		That("put $builtin:true").Puts(true),
 		// Unqualified name prefers local: to up:.
 		That("x = outer; { local:x = inner; put $x }").Puts("inner"),
 		// Unqualified name resolves to upvalue if no local name exists.
@@ -170,55 +177,70 @@ func TestVariableUse(t *testing.T) {
 	)
 }
 
+func TestVariableUse_DeprecatedSpecialNamespaces(t *testing.T) {
+	testCompileTimeDeprecation(t, "var a; put $local:a", "the local: special namespace is deprecated", 17)
+	testCompileTimeDeprecation(t, "var a; { put $up:a }", "the up: special namespace is deprecated", 17)
+	testCompileTimeDeprecation(t, "var a; { put $:a }", "the empty namespace is deprecated", 17)
+}
+
 func TestClosure(t *testing.T) {
 	Test(t,
-		That("[]{ }").DoesNothing(),
-		That("[x]{put $x} foo").Puts("foo"),
+		That("{|| }").DoesNothing(),
+		That("{|x| put $x} foo").Puts("foo"),
 
 		// Assigning to captured variable
-		That("x=lorem; []{x=ipsum}; put $x").Puts("ipsum"),
-		That("x=lorem; []{ put $x; x=ipsum }; put $x").Puts("lorem", "ipsum"),
+		That("var x = lorem; {|| set x = ipsum}; put $x").Puts("ipsum"),
+		That("var x = lorem; {|| put $x; set x = ipsum }; put $x").
+			Puts("lorem", "ipsum"),
 
 		// Assigning to element of captured variable
 		That("x = a; { x = b }; put $x").Puts("b"),
 		That("x = [a]; { x[0] = b }; put $x[0]").Puts("b"),
 
 		// Shadowing
-		That("x=ipsum; []{ local:x=lorem; put $x }; put $x").Puts("lorem", "ipsum"),
+		That("var x = ipsum; { var x = lorem; put $x }; put $x").
+			Puts("lorem", "ipsum"),
 
 		// Shadowing by argument
-		That("x=ipsum; [x]{ put $x; x=BAD } lorem; put $x").Puts("lorem", "ipsum"),
+		That("var x = ipsum; {|x| put $x; set x = BAD } lorem; put $x").
+			Puts("lorem", "ipsum"),
 
 		// Closure captures new local variables every time
-		That(`fn f []{ x=0; put []{x=(+ $x 1)} []{put $x} }
-		  {inc1,put1}=(f); $put1; $inc1; $put1
-		  {inc2,put2}=(f); $put2; $inc2; $put2`).Puts("0", 1.0, "0", 1.0),
+		That("fn f { var x = (num 0); put { set x = (+ $x 1) } { put $x } }",
+			"var inc1 put1 = (f); $put1; $inc1; $put1",
+			"var inc2 put2 = (f); $put2; $inc2; $put2").Puts(0, 1, 0, 1),
 
 		// Rest argument.
-		That("[x @xs]{ put $x $xs } a b c").Puts("a", vals.MakeList("b", "c")),
-		That("[a @b c]{ put $a $b $c } a b c d").
+		That("{|x @xs| put $x $xs } a b c").Puts("a", vals.MakeList("b", "c")),
+		That("{|a @b c| put $a $b $c } a b c d").
 			Puts("a", vals.MakeList("b", "c"), "d"),
 		// Options.
-		That("[a &k=v]{ put $a $k } foo &k=bar").Puts("foo", "bar"),
+		That("{|a &k=v| put $a $k } foo &k=bar").Puts("foo", "bar"),
 		// Option default value.
-		That("[a &k=v]{ put $a $k } foo").Puts("foo", "v"),
+		That("{|a &k=v| put $a $k } foo").Puts("foo", "v"),
+		// Option must have default value
+		That("{|&k| }").DoesNotCompile(),
+		// Exception when evaluating option default value.
+		That("{|&a=[][0]| }").Throws(ErrorWithType(errs.OutOfRange{}), "[][0]"),
+		// Option default value must be one value.
+		That("{|&a=(put foo bar)| }").Throws(
+			errs.ArityMismatch{What: "option default value", ValidLow: 1, ValidHigh: 1, Actual: 2},
+			"(put foo bar)"),
 
 		// Argument name must be unqualified.
-		That("[a:b]{ }").DoesNotCompile(),
+		That("{|a:b| }").DoesNotCompile(),
 		// Argument name must not be empty.
-		That("['']{ }").DoesNotCompile(),
-		That("[@]{ }").DoesNotCompile(),
+		That("{|''| }").DoesNotCompile(),
+		That("{|@| }").DoesNotCompile(),
 		// Option name must be unqualified.
-		That("[&a:b=1]{ }").DoesNotCompile(),
+		That("{|&a:b=1| }").DoesNotCompile(),
 		// Option name must not be empty.
-		That("[&''=b]{ }").DoesNotCompile(),
-
-		// Exception when evaluating option default value.
-		That("[&a=[][0]]{ }").Throws(ErrorWithType(errs.OutOfRange{}), "[][0]"),
-		// Option default value must be one value.
-		That("[&a=(put foo bar)]{ }").Throws(
-			errs.ArityMismatch{
-				What: "option default value", ValidLow: 1, ValidHigh: 1, Actual: 2},
-			"(put foo bar)"),
+		That("{|&''=b| }").DoesNotCompile(),
+		// Should not have multiple rest arguments.
+		That("{|@a @b| }").DoesNotCompile(),
 	)
 }
+
+func TestClosure_LegacySyntaxIsDeprecated(t *testing.T) {
+	testCompileTimeDeprecation(t, "a = []{ }", "legacy lambda syntax is deprecated", 17)
+}
