@@ -4,12 +4,14 @@ import (
 	"errors"
 	"os"
 	"os/exec"
+	"sync/atomic"
 	"syscall"
 
-	"github.com/elves/elvish/pkg/eval/vals"
-	"github.com/elves/elvish/pkg/fsutil"
-	"github.com/elves/elvish/pkg/parse"
-	"github.com/xiaq/persistent/hash"
+	"src.elv.sh/pkg/eval/errs"
+	"src.elv.sh/pkg/eval/vals"
+	"src.elv.sh/pkg/fsutil"
+	"src.elv.sh/pkg/parse"
+	"src.elv.sh/pkg/persistent/hash"
 )
 
 var (
@@ -102,5 +104,12 @@ func (e externalCmd) Call(fm *Frame, argVals []interface{}, opts map[string]inte
 		// calling `Wait` twice on a particular process object.
 		return err
 	}
+	ws := state.Sys().(syscall.WaitStatus)
+	if ws.Signaled() && isSIGPIPE(ws.Signal()) {
+		readerGone := fm.ports[1].readerGone
+		if readerGone != nil && atomic.LoadInt32(readerGone) == 1 {
+			return errs.ReaderGone{}
+		}
+	}
 	return NewExternalCmdExit(e.Name, state.Sys().(syscall.WaitStatus), proc.Pid)
 }
