@@ -4,18 +4,15 @@ import (
 	"os"
 	"testing"
 
-	. "github.com/elves/elvish/pkg/eval"
+	. "src.elv.sh/pkg/eval"
 
-	. "github.com/elves/elvish/pkg/eval/evaltest"
-	"github.com/elves/elvish/pkg/testutil"
+	. "src.elv.sh/pkg/eval/evaltest"
+	"src.elv.sh/pkg/testutil"
 )
 
 func TestBuiltinFnExternal(t *testing.T) {
-	tmpHome, cleanup := testutil.InTempHome()
-	defer cleanup()
-
-	restorePath := testutil.WithTempEnv("PATH", tmpHome+":"+os.Getenv("PATH"))
-	defer restorePath()
+	tmpHome := testutil.InTempHome(t)
+	testutil.Setenv(t, "PATH", tmpHome+":"+os.Getenv("PATH"))
 
 	Test(t,
 		That(`e = (external true); kind-of $e`).Puts("fn"),
