@@ -28,7 +28,7 @@ var typeCbMap = map[string]func(os.FileMode) bool{
 }
 
 const (
-	// noMatchOK indicates that the "nomatch-ok" glob index modifer was
+	// noMatchOK indicates that the "nomatch-ok" glob index modifier was
 	// present.
 	noMatchOK globFlag = 1 << iota
 )
@@ -63,6 +63,8 @@ var runeMatchers = map[string]func(rune) bool{
 	"upper":   unicode.IsUpper,
 }
 
+func (gp globPattern) Kind() string { return "glob-pattern" }
+
 func (gp globPattern) Index(k interface{}) (interface{}, error) {
 	modifierv, ok := k.(string)
 	if !ok {
