@@ -3,8 +3,7 @@ package eval_test
 import (
 	"testing"
 
-	. "github.com/elves/elvish/pkg/eval/evaltest"
-	"github.com/elves/elvish/pkg/eval/vars"
+	. "src.elv.sh/pkg/eval/evaltest"
 )
 
 func TestNs(t *testing.T) {
@@ -26,6 +25,6 @@ func TestNs(t *testing.T) {
 
 func TestBuiltinFunctionsReadOnly(t *testing.T) {
 	Test(t,
-		That("return~ = { }").Throws(vars.ErrSetReadOnlyVar, "return~ = { }"),
+		That("return~ = { }").DoesNotCompile(),
 	)
 }
