@@ -1,3 +1,4 @@
+//go:build !windows && !plan9
 // +build !windows,!plan9
 
 package eval
@@ -8,6 +9,7 @@ import (
 	"syscall"
 
 	"src.elv.sh/pkg/sys"
+	"src.elv.sh/pkg/sys/eunix"
 )
 
 // Process control functions in Unix.
@@ -21,7 +23,7 @@ func putSelfInFg() error {
 	// Ignoring TTOU fixes that.
 	signal.Ignore(syscall.SIGTTOU)
 	defer signal.Reset(syscall.SIGTTOU)
-	return sys.Tcsetpgrp(0, syscall.Getpgrp())
+	return eunix.Tcsetpgrp(0, syscall.Getpgrp())
 }
 
 func makeSysProcAttr(bg bool) *syscall.SysProcAttr {
