@@ -4,8 +4,8 @@ import (
 	"reflect"
 	"testing"
 
-	"github.com/elves/elvish/pkg/eval/vals"
-	"github.com/elves/elvish/pkg/glob"
+	"src.elv.sh/pkg/eval/vals"
+	"src.elv.sh/pkg/glob"
 )
 
 var reprTests = []struct {
