@@ -3,9 +3,9 @@ package eval_test
 import (
 	"testing"
 
-	. "github.com/elves/elvish/pkg/eval"
+	. "src.elv.sh/pkg/eval"
 
-	"github.com/elves/elvish/pkg/tt"
+	"src.elv.sh/pkg/tt"
 )
 
 var Args = tt.Args
