@@ -12,12 +12,11 @@ package parse
 
 import (
 	"bytes"
-	"errors"
 	"fmt"
 	"io"
 	"unicode"
 
-	"github.com/elves/elvish/pkg/diag"
+	"src.elv.sh/pkg/diag"
 )
 
 // Tree represents a parsed tree.
@@ -26,25 +25,24 @@ type Tree struct {
 	Source Source
 }
 
-// Parse parses the given source. The returned error always has type *Error
-// if it is not nil.
-func Parse(src Source) (Tree, error) {
-	return ParseWithDeprecation(src, nil)
+// Config keeps configuration options when parsing.
+type Config struct {
+	// Destination of warnings. If nil, warnings are suppressed.
+	WarningWriter io.Writer
 }
 
-// ParseWithDeprecation is like Parse, but also writes out deprecation warnings
-// to the given io.Writer.
-func ParseWithDeprecation(src Source, w io.Writer) (Tree, error) {
+// Parse parses the given source. The returned error always has type *Error
+// if it is not nil.
+func Parse(src Source, cfg Config) (Tree, error) {
 	tree := Tree{&Chunk{}, src}
-	err := ParseAs(src, tree.Root, w)
+	err := ParseAs(src, tree.Root, cfg)
 	return tree, err
 }
 
 // ParseAs parses the given source as a node, depending on the dynamic type of
-// n, writing deprecation warnings to the given io.Writer if it is not nil. If
-// the error is not nil, it always has type *Error.
-func ParseAs(src Source, n Node, w io.Writer) error {
-	ps := &parser{srcName: src.Name, src: src.Code, warn: w}
+// n. If the error is not nil, it always has type *Error.
+func ParseAs(src Source, n Node, cfg Config) error {
+	ps := &parser{srcName: src.Name, src: src.Code, warn: cfg.WarningWriter}
 	ps.parse(n)
 	ps.done()
 	return ps.assembleError()
@@ -53,13 +51,11 @@ func ParseAs(src Source, n Node, w io.Writer) error {
 // Errors.
 var (
 	errShouldBeForm               = newError("", "form")
-	errBadLHS                     = errors.New("bad assignment LHS")
 	errBadRedirSign               = newError("bad redir sign", "'<'", "'>'", "'>>'", "'<>'")
 	errShouldBeFD                 = newError("", "a composite term representing fd")
 	errShouldBeFilename           = newError("", "a composite term representing filename")
 	errShouldBeArray              = newError("", "spaced")
 	errStringUnterminated         = newError("string not terminated")
-	errChainedAssignment          = newError("chained assignment not yet supported")
 	errInvalidEscape              = newError("invalid escape sequence")
 	errInvalidEscapeOct           = newError("invalid escape sequence", "octal digit")
 	errInvalidEscapeHex           = newError("invalid escape sequence", "hex digit")
@@ -72,6 +68,7 @@ var (
 	errShouldBeRParen             = newError("", "')'")
 	errShouldBeCompound           = newError("", "compound")
 	errShouldBeEqual              = newError("", "'='")
+	errShouldBePipe               = newError("", "'|'")
 	errBothElementsAndPairs       = newError("cannot contain both list elements and map pairs")
 	errShouldBeNewline            = newError("", "newline")
 )
@@ -344,6 +341,30 @@ const (
 	Append
 )
 
+// Filter is the Elvish filter DSL. It uses the same syntax as arguments and
+// options to a command.
+type Filter struct {
+	node
+	Args []*Compound
+	Opts []*MapPair
+}
+
+func (qn *Filter) parse(ps *parser) {
+	parseSpaces(qn, ps)
+	for {
+		r := ps.peek()
+		switch {
+		case r == '&':
+			ps.parse(&MapPair{}).addTo(&qn.Opts, qn)
+		case startsCompound(r, NormalExpr):
+			ps.parse(&Compound{}).addTo(&qn.Args, qn)
+		default:
+			return
+		}
+		parseSpaces(qn, ps)
+	}
+}
+
 // Compound = { Indexing }
 type Compound struct {
 	node
@@ -402,9 +423,9 @@ func startsCompound(r rune, ctx ExprCtx) bool {
 // Indexing = Primary { '[' Array ']' }
 type Indexing struct {
 	node
-	ExprCtx  ExprCtx
-	Head     *Primary
-	Indicies []*Array
+	ExprCtx ExprCtx
+	Head    *Primary
+	Indices []*Array
 }
 
 func (in *Indexing) parse(ps *parser) {
@@ -414,7 +435,7 @@ func (in *Indexing) parse(ps *parser) {
 			ps.error(errShouldBeArray)
 		}
 
-		ps.parse(&Array{}).addTo(&in.Indicies, in)
+		ps.parse(&Array{}).addTo(&in.Indices, in)
 
 		if !parseSep(in, ps, ']') {
 			ps.error(errShouldBeRBracket)
@@ -456,10 +477,12 @@ type Primary struct {
 	node
 	ExprCtx ExprCtx
 	Type    PrimaryType
+	// Legacy lambda uses [args]{ body } instead of { |args| body }
+	LegacyLambda bool
 	// The unquoted string value. Valid for Bareword, SingleQuoted,
 	// DoubleQuoted, Variable, Wildcard and Tilde.
 	Value    string
-	Elements []*Compound // Valid for List and Labda
+	Elements []*Compound // Valid for List and Lambda
 	Chunk    *Chunk      // Valid for OutputCapture, ExitusCapture and Lambda
 	MapPairs []*MapPair  // Valid for Map and Lambda
 	Braced   []*Compound // Valid for Braced
@@ -781,6 +804,7 @@ items:
 		ps.error(errShouldBeRBracket)
 	}
 	if parseSep(pn, ps, '{') {
+		pn.LegacyLambda = true
 		pn.lambda(ps)
 	} else {
 		if loneAmpersand || len(pn.MapPairs) > 0 {
@@ -798,6 +822,28 @@ items:
 // lambda parses a lambda expression. The opening brace has been seen.
 func (pn *Primary) lambda(ps *parser) {
 	pn.Type = Lambda
+	if !pn.LegacyLambda {
+		parseSpacesAndNewlines(pn, ps)
+		if parseSep(pn, ps, '|') {
+			parseSpacesAndNewlines(pn, ps)
+		items:
+			for {
+				r := ps.peek()
+				switch {
+				case r == '&':
+					ps.parse(&MapPair{}).addTo(&pn.MapPairs, pn)
+				case startsCompound(r, NormalExpr):
+					ps.parse(&Compound{}).addTo(&pn.Elements, pn)
+				default:
+					break items
+				}
+				parseSpacesAndNewlines(pn, ps)
+			}
+			if !parseSep(pn, ps, '|') {
+				ps.error(errShouldBePipe)
+			}
+		}
+	}
 	ps.parse(&Chunk{}).addAs(&pn.Chunk, pn)
 	if !parseSep(pn, ps, '}') {
 		ps.error(errShouldBeRBrace)
@@ -809,7 +855,7 @@ func (pn *Primary) lambda(ps *parser) {
 func (pn *Primary) lbrace(ps *parser) {
 	parseSep(pn, ps, '{')
 
-	if r := ps.peek(); r == ';' || r == '\r' || r == '\n' || IsInlineWhitespace(r) {
+	if r := ps.peek(); r == ';' || r == '\r' || r == '\n' || r == '|' || IsInlineWhitespace(r) {
 		pn.lambda(ps)
 		return
 	}
