@@ -4,7 +4,7 @@ package parseutil
 import (
 	"strings"
 
-	"github.com/elves/elvish/pkg/parse"
+	"src.elv.sh/pkg/parse"
 )
 
 // FindLeafNode finds the leaf node at a specific position. It returns nil if
@@ -25,7 +25,7 @@ descend:
 
 // Wordify turns a piece of source code into words.
 func Wordify(src string) []string {
-	tree, _ := parse.Parse(parse.Source{Name: "[wordify]", Code: src})
+	tree, _ := parse.Parse(parse.Source{Code: src}, parse.Config{})
 	return wordifyInner(tree.Root, nil)
 }
 
