@@ -3,10 +3,11 @@ package ui
 import (
 	"bytes"
 	"fmt"
+	"math/big"
 	"strconv"
 
-	"github.com/elves/elvish/pkg/eval/vals"
-	"github.com/elves/elvish/pkg/wcwidth"
+	"src.elv.sh/pkg/eval/vals"
+	"src.elv.sh/pkg/wcwidth"
 )
 
 // Text contains of a list of styled Segments.
@@ -61,12 +62,12 @@ func (t Text) Index(k interface{}) (interface{}, error) {
 	}
 }
 
-// Concat implements Text+string, Text+float64, Text+Segment and Text+Text.
+// Concat implements Text+string, Text+number, Text+Segment and Text+Text.
 func (t Text) Concat(rhs interface{}) (interface{}, error) {
 	switch rhs := rhs.(type) {
 	case string:
 		return Concat(t, T(rhs)), nil
-	case float64:
+	case int, *big.Int, *big.Rat, float64:
 		return Concat(t, T(vals.ToString(rhs))), nil
 	case *Segment:
 		return Concat(t, Text{rhs}), nil
@@ -77,12 +78,12 @@ func (t Text) Concat(rhs interface{}) (interface{}, error) {
 	return nil, vals.ErrConcatNotImplemented
 }
 
-// RConcat implements string+Text and float64+Text.
+// RConcat implements string+Text and number+Text.
 func (t Text) RConcat(lhs interface{}) (interface{}, error) {
 	switch lhs := lhs.(type) {
 	case string:
 		return Concat(T(lhs), t), nil
-	case float64:
+	case int, *big.Int, *big.Rat, float64:
 		return Concat(T(vals.ToString(lhs)), t), nil
 	}
 
