@@ -3,8 +3,8 @@ package ui
 import (
 	"testing"
 
-	"github.com/elves/elvish/pkg/eval/vals"
-	"github.com/elves/elvish/pkg/tt"
+	"src.elv.sh/pkg/eval/vals"
+	"src.elv.sh/pkg/tt"
 )
 
 var Args = tt.Args
