@@ -67,11 +67,16 @@ func contentIs(what string) string {
 		what)
 }
 
+const fontFaceTemplate = `@font-face { font-family: %v; font-weight: %v; font-style: %v; font-stretch: normal; font-display: block; src: url("%v/fonts/%v.woff2") format("woff");}`
+
 func newTemplate(name, root string, sources ...string) *template.Template {
 	t := template.New(name).Funcs(template.FuncMap(map[string]interface{}{
 		"is":      func(s string) bool { return s == name },
 		"rootURL": func() string { return root },
 		"getEnv":  os.Getenv,
+		"fontFace": func(family string, weight int, style string, fname string) string {
+			return fmt.Sprintf(fontFaceTemplate, family, weight, style, root, fname)
+		},
 	}))
 	for _, source := range sources {
 		template.Must(t.Parse(source))
